#!/usr/bin/env python

# parts of this shamelessly stolen from radix's excellent release-twisted program

"""
this is the release script for all divmod products

this script should be run as the 'release' user on tesla. It assumes that you have
checked out the *entire* *repository* to Projects/Quotient (using svn co file:///svn/Quotient)

The required --ver argument should be of the form N.N.N{rcN,alphaN,}   

in order to run the 'new user with a clean environment' tests you need to have sudo
installed, and this script makes some assumptions about the permmissions that the
'release' user (the user running this script) and the 'test' user (the "new" user)
have in relation to each other.

your sudoeers should have a line like the following:

release        ALL=(root) /bin/su

also your /etc/group file should have a line like

release:x:1001:
test:x:1002:release

and the /home/test directory should be group-writable
"""

import os, os.path as osp, sys, shutil, re, warnings
from os.path import join as opj, expanduser as xu

from twisted.python import usage, util, failure

import cgitb  # more verbose tracebacks
cgitb.enable(display=1, format='text')

defaultSVNBase = "svn+ssh://divmod.org/svn"
debugSVNBase = opj(defaultSVNBase, 'Quotient/sandbox/slyphon/debug-release-quotient')
localSVNBase = 'file:///svn/Quotient'
changeVersInfoFor = {'Quotient': ['README', 'quotient/copyright.py', 'setup.py']}
repowc = xu("~/Projects/Quotient")


debug = False


class DirectoryExists(OSError):
    """Some directory exists when it shouldn't."""
    pass

class DirectoryDoesntExist(OSError):
    """Some directory doesn't exist when it should."""
    pass

class CommandFailed(OSError):
    pass

# utilities
def sh(command):
    """
    I'll try to execute `command', and if `sensitive' is true, I'll
    ask before running it.  If the command returns something other
    than 0, I'll raise CommandFailed(command).
    """
    print "sh called with arg: %s" % command
    if os.system(command) != 0:
        raise CommandFailed(command)

def cdAndRun(dir, f, *a, **kw):
    """guarantees as much as possible that after we cd and run f, that we will return to the
    original working directory """
    wd = osp.abspath(os.getcwd())
    try:
        os.chdir(dir)
        f(*a, **kw)
    finally:
        os.chdir(wd)


class SVN(object):
    def cp(self, here, there, m=None):
        sh("svn cp -m '%s' %s %s" % (m, here, there))

    def rm(self, path, m=None):
        sh("svn rm -m '%s' %s" % (m, path))

    def mv(self, here, there, m=None):
        sh("svn mv -m '%s' %s %s" % (m, here, there))

    def commit(self, pathlist=[], m=None):
        sh('svn commit -m "%s" %s' % (m, ''.join(pathlist)))

    def update(self, path):
        cdAndRun(path, sh, "svn update %s" % path)

    def purge(self, path):
        from popen2 import Popen3
        p = Popen3('svn status %s' % path)

        print "going to remove files in %s" % path
        for s in p.fromchild.read().split('\n'):
            if s and s.startswith('?'):
                topurge = s.split()[1]
                print topurge
                sh('rm -rf %s' % topurge)

# only need one of these
svn = SVN()

restr = r"(?P<basever>\d\.\d{1,}\.\d*)(?P<type>(?:rc|alpha){0,1})(?P<tinyver>\d*)"
reVersion = re.compile(restr)

class ReleaseDivmod(object):
    # o/` Release MEEEEEEEE! o/`

    ballname = None

    def __init__(self, opts):
        self.opt = opts
        self.__dict__.update(opts)

    def do(self, cmdName):
        print 'do called with arg: %s' % cmdName

        # all methods are called after first switching to the home directory
        # to ensure consistent filesystem navigation
        cdAndRun(xu('~'), getattr(self, cmdName))

    def copyTrunkToWIP(self):
        if osp.exists(self.wip):
            svn.rm(self.wip, m="Removing wip/wip-release-%s in order to re-copy from trunk" % self.fullver)

        svn.update(self.releasefrom)
        svn.cp(self.releasefrom, self.wip, m="Copying %s to wip-release-%s branch." % (self.releasefrom, self.fullver))
        
    def updateVersion(self):
        # XXX: brokenness! this will only work for Quotient atm!
        if self.product != 'Quotient':
            warnings.warn("Auto-updating version not implemented yet for any product but Quotient!")

        oldver = 'trunk-HEAD'
        newver = self.fullver

        wd = os.getcwd()
        try:
            os.chdir(self.wip)
            for f in changeVersInfoFor[self.product]:
                foundOldver = False
                print "checking %s for old version" % f

                newf = file("%s.new" % f, 'w')
                for line in file(f, 'r'):
                    if oldver in line:
                        foundOldver = True
                    newf.write(line.replace(oldver, newver))
                if foundOldver:
                    sh('mv %s.new %s' % (f, f))
                else:
                    sh('rm %s.new' % f)
        finally:
            os.chdir(wd)


    def makeBalls(self):
        cdAndRun(self.wip, sh, '%s setup.py sdist' % sys.executable)

    def testBalls(self):
        o, wd = self.opt, os.getcwd()
        bp = "/tmp/test_bashrc"
        origbp = opj(self.testhome, '.bash_profile')

        try:
            os.chdir(self.wip)
            sh('cp dist/%s.tar.gz /tmp' % self.rname)

            try:
                # write the .bash_profile we want to use for this test
                bpfp = file(bp, 'w')
                bpfp.write("""
export PYTHONPATH=~/lib/python2.3/site-packages
export PATH=~/bin:"${PATH}"
""")
                bpfp.flush(); bpfp.close()

                # now move the test user's .bash_profile out of the way temporarily
                sh("mv -v %s %s.bak" % (origbp, origbp))

                # copy our .bash_profile into position
                sh("cp -vf %s %s" % (bp, origbp))

                # copy the test script into position
                sh("cp -f %s %s" % (self.testscriptpath, self.testhome))

                sh("sudo su - %s '%s %s %s'" % (self.testuser, sys.executable, opj(self.testhome, self.testscript), self.testargs))

            finally:
                # replace the test user's original .bash_profile
                sh("mv %s.bak %s" % (origbp, origbp))
                if osp.exists(bp):
                    sh('rm %s' % bp)
        finally:
            os.chdir(wd)

    def resetDebug(self):
        raise NotImplementedError, "haven't gotten around to this yet"


    def finalizeTag(self):
        if not osp.exists(opj('tmp', self.ballname)):
            sh('cp %s %s' % (opj(self.wip, 'dist', self.ballname), opj('/tmp', self.ballname)))

        svn.commit([self.wip], m="committing to wip-release-%s before finalizing tag" % self.fullver)

        # get rid of all locally-modified files we didn't add
        svn.purge(self.wip)

        svn.mv(self.wip, self.tag, m="Finalizing release of %s" % self.fullver)
        svn.commit([self.wip, self.tag], m="%s release of %s" % (self.fullver, self.product))


    def releaseBalls(self):
        o, wd = self.opt, os.getcwd()

        sh('cp %s %s' % (opj('/tmp', self.ballname), self.releaseto))

        print "WOOHOO!"

    def createMD5Sum(self):
        assert self.ballname 
        assert self.releaseto
        import md5 
        m = md5.new()
        ballpath = opj(self.releaseto, self.ballname)
        for line in file(ballpath):
            m.update(line)

        fp = file('%s.MD5' % ballpath, 'w')
        fp.write("%s  %s " % (m.hexdigest(), self.ballname))
        fp.flush(); fp.close()


    def updateSites(self):
        reDev = re.compile(r'(?P<devtar>Quotient-\d\.\d{1,}\.\d*(?:rc|alpha)\d*.tar.gz)')
        dlpage = opj(self.sites_dir, 'divmod.org/gen/source/Home/Downloads.html')

        svn.update(osp.dirname(dlpage))

        dlpagenew = "%s.new" % dlpage
        dlpn = file(dlpagenew, 'w')

        txt = file(dlpage, 'r').read()

        m = reDev.search(txt)
        newtxt = reDev.sub(self.rname, txt)
        dlpn.write(newtxt); dlpn.close()

        if not m:
            sh('rm %s' % dlpagenew)
            warnings.warn('Did not find old version in %s, not committing' % (dlpage))
            return 

        sh('mv %s %s' % (dlpagenew, dlpage))
        if not self.dry_run:
            svn.commit([dlpage], m="Updated divmod.org's download page to reflect release %s" % self.rname)
       


class Options(usage.Options):
    optParameters = [
        ['product', 'p', None,
         '(required) the Divmod product to release {Quotient, Lupy, Nevow}'],
        ['ver', 'v', None,
         "(required) The version of this release"],
        ['commands', None, None,
         "(optional) The individual steps to run; You should only use "
         "this if something broke"],
        ['qotrfile', None, None,
         "(optional) The filename containing the QOTR to use."],
        ['repo', None, localSVNBase,
         "The BASE of the SVN repository. trunk, branches/foo, etc, will "
         "automatically be added"],
        ['repowc', None, repowc,
         "The base of the local checkout of the repository you wish to release"],
        ['releaseto', None, xu('~/public_html/divmod/'),
         "The directory to put tarballs."],
        ['releasefrom', None, opj(repowc, 'trunk'),
         "where the release should be cut from. will automatically svn update that dir"],
        ['sites_dir', None, xu("~/Projects/DivmodSites/trunk"),
         "location of DivmodSites/trunk for auto-updating divmod.org site"],
        ['start_with', None, None,
         "(developers) if the command sequence fails somewhere, start the sequence at step <command>"],
        ['testscript', 't', None,
         "path to the test script the testuser should run"],
        ['testuser', None, 'test',
         "NOTE: this requires setup of sudo to allow you to switch uid's to testuser\n"
         "see release_divmod's module docstring for more details"], 
        ["testgroup", None, 'test',
         "the group that testuser belongs to, defaults to same value as --testuser argument"],
        ['testhome', None, None,
         "the testuser's home directory (defaults to /home/--testuser)"],
        ['reset_debug', None, None,
         "developer use only!"]
        ]

    optFlags = [
        ['dry_run', 'n',
         "Don't do anything 'permanent': finalize tags, upload tarballs, etc."],
        ['debug', 'b',
         'use the debug repository, and be verbose, and yadda yadda...'],
        ['sites', 's',
         "update DivmodSites to reflect new release"]
        ]


    def _makeAssertions(self):
        def _mustHave(keys, failMsg=''):
            for k in keys:
                if not self[k]:
                    if not failMsg:
                        raise usage.UsageError("--%s is required!" % k)
                    else:
                        raise usage.UsageError(failMsg)

        def _mustBe(key, okVals):
            if self[key] not in okVals:
                print 'key: %s, self[key]: %s, okVals: %s' % (key, self[key], okVals)
                raise usage.UsageError("--%s must be one of %s" % (self[key], okVals))

        if self['reset_debug']:
            return

        mh = ['product', 'ver']

        if self['sites'] and (not osp.exists(self['sites_dir']) or not osp.isdir(self['sites_dir'])):
            raise RuntimeError('specified argument "%s" to --sites_dir does not exist!' % self['sites_dir'])

        _mustHave(mh)

        _mustBe('product', ['Quotient', 'Nevow', 'Lupy'])



    def _makeMeConvenient(self):
        if debug:
            self['repo'] = opj(debugSVNBase, self['product'])
        else:
            self['repo'] = opj(defaultSVNBase, self['product'])
 
        m = reVersion.search(self['ver'])

        self.update(m.groupdict())  # heh, heh

        self['fullver'] = ''.join([self['basever'], self['type'], self['tinyver']])

        self['rname'] = '%s-%s' % (self['product'], self['fullver'])

        def _k(key, *args):
            # how bout a big hand for functional programming, folks?
            self[key] = opj(*args)

        _k('trunk', repowc, 'trunk')
        _k('wip',  repowc, 'branches/wip', 'wip-release-%s' % self['fullver'])
        _k('tag', repowc, 'tags', 'releases', self['fullver'])

        self['temptag'] = self['tag'] + '-TEMP'

        self['ballname'] = '%s.tar.gz' % self['rname']

        if self['testscript']:
            self['testscriptpath'] = osp.abspath(self['testscript'])
            if not self['testhome']:
                self['testhome'] = opj('/home', self['testuser'])
            self['testargs'] = "--localQball --q-ver=%s --product=%s" % (self['fullver'], self['product'])

        if self['debug']:
            l = [(k, v) for k, v in self.iteritems()]
            l.sort()
            for k, v in l:
                print 'k: %s\tv: %s' % (k,v)


    def _inferCommands(self):
        """infer commands to run 
        This code expresses the high-level workflow of the
        release procedure."""

        cmds = []
        if self['commands'] is not None:
            cmds = self['commands'].split(',')

        elif self['reset_debug']:
            cmds = ['resetDebug']

        else:
            cmds.extend(['copyTrunkToWIP', 'updateVersion', 'makeBalls'])

            if self['testscript']:
                cmds.append('testBalls')

            if not self['dry_run']:
                cmds.extend(['finalizeTag', 'releaseBalls', 'createMD5Sum'])

            if self['sites']:
                cmds.append('updateSites')

            if self['start_with']:
                while len(cmds) > 0:
                    if cmds[0] == self['start_with']:
                        break
                    cmds.pop(0)
                    

        self['commands'] = cmds

        print "GOING TO DO", cmds


    def postOptions(self):
        if self['debug'] or self['reset_debug']:
            global debug
            debug = True

        self._makeAssertions()
        self._makeMeConvenient()
        self._inferCommands()



def main():
    try:
        opts = Options()
        opts.parseOptions()
    except usage.UsageError, ue:
        print "%s: %s (see --help)" % (sys.argv[0], ue)
        sys.exit(2)
#    sys.exit("BYE")

    rd = ReleaseDivmod(opts)

    for cmd in opts['commands']:
        rd.do(cmd)


if __name__=='__main__':
    main()

