import pprint
from atop.store import Pool
from atop.credup import preauth
from twisted.spread import pb
from twisted.python.components import Adapter
from twisted.persisted.styles import Ephemeral
from twisted.python.reflect import qual
from twisted.manhole.service import runInConsole

class PortholeError(Exception):
    pass

def wrap(item):
    if isinstance(item, Pool):
        return PoolRef(item)
    else:
        return ItemRef(item)

class PortholePerspective(pb.Avatar, Adapter, Ephemeral):    

    def __init__(self, original):
        self.avatar = original
        
    def perspective_getRootPool(self):
        def _():
            return PoolRef(self.avatar.store.rootRef.getItem())
        return self.avatar.store.transact(_)

    def perspective_selectUser(self, username):
        def _():
            ava = preauth(self.avatar.store, username)
            return (PoolRef(ava.getRootPool()), ava.storeID)
        return self.avatar.store.transact(_)

    def perspective_select(self, storeid):
        def _():
            return wrap(self.avatar.store.getItemByID(storeid))
        return self.avatar.store.transact(_)
        
    

class ItemRef(pb.Referenceable, Ephemeral):
    
    def __init__(self, item):
        self.item = item
        self.namespace = {'self': self.item}
    def remote_name(self):
        return self.item.name

    def remote_scrutinize(self):
        def _():
            return "Class: %s\nName: %s\n%s" % (self.item.__class__, self.item.name, pprint.pformat(self.item.__dict__))
        return self.item.store.transact(_)
    def remote_list(self):
        raise PortholeError, "Not a pool"

    def remote_eval(self, code):
        def _():
            val = runInConsole(code, None, globalNS=self.namespace)
            self.item.touch()
            if val is not None:
                self.namespace["_"] = val
            return str(val)
        self.item.store.transact(_)

class PoolRef(ItemRef):

    def remote_indices(self):
        pass

    def remote_list(self):
        def _():
            return [(x.storeID, wrap(x), x.name, qual(x.__class__)) for x in self.item.queryIndex()]
        return self.item.store.transact(_)
