# -*- test-case-name: atop.test.test_store -*-
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

"""
Helper module for allowing both cPickle and pickle to be used.
"""

from twisted.python import context, reflect

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

PROFILING = False

class Reverter(object):
    def __new__(self):
        return context.get('item')

try:
    import cPickle as pickle

    def unpickleItem(stringPickle):
        """Unpickle an item from a string stored in the database, doing revert
        magic if appropriate.
        """
        u = pickle.Unpickler(StringIO(stringPickle))
        item = context.get('item')
        if item is not None:
            def makeReverter(iModule, iClass):
                del u.find_global
                return Reverter
            u.find_global = makeReverter
            forgottenAttributes = []
            reflect.accumulateClassList(item.__class__, 'persistenceForgets', forgottenAttributes)
            oldAttributes = {}
            for attr in forgottenAttributes:
                if attr in item.__dict__:
                    value = item.__dict__[attr]
                    oldAttributes[attr] = value
            i = u.load()
            assert i is item
            i.reactivate(oldAttributes)
            return i
        else:
            return u.load()

except ImportError:
    import pickle

    def load_global(self):
        self.readline() # Module name
        self.readline() # Class name
        self.dispatch[pickle.GLOBAL] = pickle.Unpickler.dispatch[pickle.GLOBAL]
        self.append(Reverter)

    def unpickleItem(stringPickle):
        """Unpickle an item from a string stored in the database, doing revert
        magic if appropriate.
        """
        u = pickle.Unpickler(StringIO(stringPickle))
        item = context.get('item')
        if item is not None:
            u.dispatch = pickle.Unpickler.dispatch.copy()
            u.dispatch[pickle.GLOBAL] = load_global
            forgottenAttributes = []
            reflect.accumulateClassList(item.__class__, 'persistenceForgets', forgottenAttributes)
            oldAttributes = {}
            for attr in forgottenAttributes:
                if attr in item.__dict__:
                    value = item.__dict__[attr]
                    oldAttributes[attr] = value
            i = u.load()
            assert i is item
            i.reactivate(oldAttributes)
            return i
        else:
            return u.load()

if PROFILING:
    def pickleItem(*a,**kw):
        return pickle.dumps(*a,**kw)
else:
    pickleItem = pickle.dumps

PICKLE_PROTOCOL = pickle.HIGHEST_PROTOCOL
