# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from atop.powerup import Powerup, IPowerStation, Storeup
from atop.store import Item, Pool, Store, SubStore

from atop.test import unittest

from os.path import join as opj

class FooUp(Powerup):
    powerupName = 'Foo'
    called = 0
    def setUpPools(self, substore):
        FooUp.called += 1

class BarUp(Powerup):
    powerupName = 'Bar'
    called = 0
    def setUpPools(self, substore):
        BarUp.called += 1

class QuxUp(Powerup):
    powerupName = 'Qux'
    dependencies = [FooUp, BarUp]
    called = 0
    def setUpPools(self, substore):
        QuxUp.called += 1

class PowerupTest(unittest.TestCase):

    def dbSetUp(self):
        self.substore = SubStore(self.store)

    def assertify(self):
        for v in [__name__ + '.FooUp',
                  __name__ + '.BarUp',
                  __name__ + '.QuxUp']:
            self.failUnlessIn(v, IPowerStation(self.substore).powerups.keys())
        self.assertEquals(FooUp.called, 1)
        self.assertEquals(BarUp.called, 1)
        self.assertEquals(QuxUp.called, 1)

    def partDependencies1(self):
        self.assertEquals( IPowerStation(self.substore).powerups , {} )
        IPowerStation(self.substore).installPowerup(FooUp)
        
        self.failUnlessIn(__name__ + '.FooUp', IPowerStation(self.substore).powerups)
        
        IPowerStation(self.substore).installPowerup(QuxUp)
        self.assertify()

    def partDependencies2(self):
        self.assertify()

    multitxnDependencyTest = [partDependencies1, partDependencies2]

class QuuxUp(Storeup):
    mangled = 0

from twisted.trial import unittest as trialtest

class StoreupTest(trialtest.TestCase):
    def testInstallAndFind(self):
        p = self.mktemp()
        openit = lambda : Store(opj(p,'db'), opj(p,'files'))
        s = openit()
        def tx1():
            qu1 = IPowerStation(s).installPowerup(QuuxUp)
            qu1.mangled = 1
        s.transact(tx1)
        s.close()
        s = openit()
        def tx2():
            qu2 = IPowerStation(s).installPowerup(QuuxUp)
            self.failUnless(qu2.mangled)
        s.transact(tx2)
        s.close()
