# -*- test-case-name: atop.test.test_service -*-
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

"""
Tests for atop.service
"""

from twisted.application.service import IService, IServiceCollection

from atop.test import unittest
from atop.service import Serviceup
from atop.powerup import Storeup

class StoreService(Storeup):
    powerupName = 'test service'

    def setUpPools(self, st):
        self.called = {}

    def privilegedStartService(self):
        self.called['privilegedStartService'] = True

    def startService(self):
        self.called['startService'] = True

    def stopService(self):
        self.called['stopService'] = True

class Service(unittest.TestCase):
    storePowerups = [Serviceup]

    def dbtestComponents(self):
        IService(self.store)
        IServiceCollection(self.store)

    def dbtestService(self):
        IService(self.store).startService()
        self.failUnless(IService(self.store).running)
        IService(self.store).stopService()
        self.failIf(IService(self.store).running)

    def dbtestUnstartedServiceCollection(self):
        svcoll = IServiceCollection(self.store)
        svc = StoreService(self.store)
        svcoll.addService(svc)

        self.assertEquals(list(svcoll), [svc])
        self.assertIdentical(svcoll.getServiceNamed("test service"), svc)

        self.assertRaises(RuntimeError, svcoll.addService, svc)

        svcoll.removeService(svc)
        self.assertEquals(list(svcoll), [])

        self.assertRaises(ValueError, svcoll.removeService, svc)

        self.failIf(svc.called.get('privilegedStartService'))
        self.failIf(svc.called.get('startService'))
        self.failIf(svc.called.get('stopService'))

    def dbtestStartedServiceCollection(self):
        svcoll = IServiceCollection(self.store)
        svcoll.startService()

        svc = StoreService(self.store)
        svcoll.addService(svc)

        self.failUnless(svc.called.get('startService'))

        self.assertEquals(list(svcoll), [svc])
        self.assertIdentical(svcoll.getServiceNamed("test service"), svc)

        self.assertRaises(RuntimeError, svcoll.addService, svc)

        svcoll.removeService(svc)
        self.assertEquals(list(svcoll), [])

        self.failUnless(svc.called.get('stopService'))

        self.assertRaises(ValueError, svcoll.removeService, svc)
