
from twisted.python import log

from atop.test import unittest
from atop import store
from atop import sched
from atop import upgrade

def stopReactor(ignored):
    from twisted.internet import reactor
    reactor.crash()

class ReindexTestCase(unittest.TestCase):
    storePowerups = [sched.SystemScheduler]

    def testStringifyUpgradeException(self):
        str(upgrade.QueuedTaskError("It is erroneous"))

    def dbtestSimpleReindex(self):
        p = store.Pool(self.store, "Test Pool")
        items = [store.Item(self.store, str(i).zfill(3)) for i in range(2)]
        map(p.addItem, items)

        p.addIndex(store.NameIndex())
        self.assertEquals(list(p.queryIndex('name')), [])

        d = upgrade.upgradeIndex(p, "name")
        from twisted.internet import reactor
        d.addCallback(stopReactor)
        reactor.run()

        self.assertEquals(list(p.queryIndex('name')), items)

    def dbtestSomeOverlap(self):
        p = store.Pool(self.store, "Test Pool")
        items = [store.Item(self.store, str(i).zfill(3)) for i in range(2)]
        map(p.addItem, items)

        p.addIndex(store.NameIndex())
        self.assertEquals(list(p.queryIndex('name')), [])

        newItems = [store.Item(self.store, str(i).zfill(3)) for i in range(2, 4)]
        map(p.addItem, newItems)

        self.assertEquals(list(p.queryIndex('name')), newItems)

        d = upgrade.upgradeIndex(p, "name")
        d.addCallback(stopReactor)
        d.addErrback(log.err)
        from twisted.internet import reactor
        reactor.run()

        self.assertEquals(list(p.queryIndex('name')), items + newItems)

class CounterTestCase(unittest.TestCase):
    storePowerups = [sched.SystemScheduler]

    # skip = "These hang for buildbot."

    def dbtestSimpleCount(self):
        p = store.Pool(self.store)
        p.addIndex(store.NameIndex('x'))
        p.addIndex(store.NameIndex('y'))
        p.addIndex(store.NameIndex('z'))

        for idxs in (('x', 'y', 'z'), ('x', 'y'), ('x',)):
            for i in range(3):
                i = store.Item(self.store, str(i))
                for iname in idxs:
                    i.setIndexKey(iname, str(i))
                p.addItem(i)

        for iname in p.indexCounter:
            p.indexCounter[iname] = 0

        d = upgrade.recountIndexes(p)
        unittest.deferredResult(d)

        for name, count in ('x', 9), ('y', 6), ('z', 3), ('uid', 9):
            self.assertEquals(p.indexCounter[name], count)
