# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import os
import string

from atop.powerup import Powerup
from atop.tpython import Forgetter
from atop.store import IPipe, Pool, NameIndex, CaseInsensitiveIndex, referenced

from lupy.indexer import Index

from mimemessage import  ICleanPoint, EmailPowerup

from twisted.python import log
from twisted.internet import threads, reactor
from twisted.internet import defer
from twisted.python.components import Interface, implements

from stats import Statoscope

from quotient import taglib
from quotient.web.root import Base
from quotient.web import pools
from quotient.proto.smtpout import IOutgoingMail, OutgoingMailup
from quotient.prefup import touchNForward
from quotient.application import IPeopleApp, PeopleApp



class IAddressBook(Interface):
    def search(self, query):
        pass

    
class AddressBook(Powerup):
    __implements__ = IPipe, IAddressBook, Powerup.__implements__,
    dependencies = [PeopleApp]
    powerupName = 'Address Book'

    addressPool = referenced

    persistenceVersion = 3

    def upgradeToVersion3(self):
        self.touch()
        self.addressPool = self.addressPoolRef.getItem()
        del self.addressPoolRef

    def upgradeToVersion2(self):
        self.touch()
        self.addressPoolRef.getItem().addIndex(CaseInsensitiveIndex("lastName"))

    def upgradeToVersion1(self):
        # TODO - Get rid of notey the Notebook ref
        from quotient.prefup import IPrefs
        ignore = IPrefs(self.store)
        app = IPeopleApp(self.store)
        self.touch()
        app.appRoot.addItem(self.addressPoolRef.getItem())
        self.store.getRootPool().removeItem(self.addressPoolRef.getItem())

    # TODO - Provisioning and Statoscope
    def setUpPools(self, avatar):
        avatar.setComponent(IAddressBook, self)
        p = Pool(avatar, name='Address Book')
        p.addIndex(NameIndex())
        p.addIndex(CaseInsensitiveIndex('lastName'))
        self.addressPool = p
        app = IPeopleApp(avatar)
        app.appRoot.addItem(p)
        self.invisible = True

    def addPerson(self, person):
        """
        We have to find a better way of doing this.
        Until then, some brittle code.
        """
        def _():
            self.addressPool.addItem(person)
        self.store.transact(_)

    def removePerson(self, person):
        """
        We have to find a better way of doing this.
        Until then, some brittle code.
        """
        def _():
            self.addressPool.removeItem(person)
        self.store.transact(_)
