
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.


from twisted.cred import portal
from twisted.cred.checkers import AllowAnonymousAccess, ICredentialsChecker
#from twisted.web.woven import guard
from twisted.application import service, strports
from twisted.python import usage

from atop.store import Store, StorageService
from atop.sched import SubScheduler, SchedulerService
from atop.powerup import IPowerStation
from atop import regadapt
from atop.credup import StoreRealm, FwdRealm, FwdChecker

from quotient.signup import BoxOffice
from quotient.sysup import Sysup, makeUser, Adminup, ControlPanel, IControlPanel
from quotient.searchup import Searchup
from quotient.prefup import Prefup
from quotient.backup import Backup
from quotient.meterer import MeterStoreup
from quotient.tasks import Taskmaster

from nevow import guard
from quotient.web import root
from quotient.web import utility
from quotient.web.webup import WebPowerup


class Options(usage.Options):
    optParameters = [
        ['webport','w', '8485','Port to run web server on.'],
        ['datadir','d', 'AnvilService/db', 'Directory name to put database in.'],
        ['msgcal', 'm', 'AnvilService/mail', 'Directory to put original email messages in.'],
        ['domain', '', 'localhost', 'Domain to run services on.'],
    ]

    def __init__(self, *a, **b):
        usage.Options.__init__(self,*a,**b)
        self.users = []                 # [(user, password, dirs, mboxes)]

    def opt_user(self, user):
        """a username:password combination to create"""
        u, p = user.split(":")
        self.users.append((u,p,[],[]))

ALL = (Prefup, WebPowerup, SubScheduler, Searchup, Taskmaster)

VISIBLE = (Taskmaster,)

POWERUPS = [(pup, pup in VISIBLE) for pup in ALL]

# Only installed for users created on the command line
TAP_PUPS = [
    (Adminup, False)
    ]

STOREUPS = (StoreRealm, BoxOffice, ControlPanel, Backup, MeterStoreup)


def makeService(config):
    stor = Store(config['datadir'], config['msgcal'])

    s = service.MultiService()

    storeService = StorageService(stor)
    storeService.setServiceParent(s)


    def _():
        sScheduler = SchedulerService(stor)
        ps = IPowerStation(stor)
        for storeup in STOREUPS:
            ps.installPowerup(storeup)
        sRealm = FwdRealm(portal.IRealm(stor).referenceTo())
        sChecker = FwdChecker(ICredentialsChecker(stor).referenceTo())
        # Non-transactional code follows.
        webPortal = portal.Portal(sRealm)
        webPortal.registerChecker(sChecker)
        webPortal.registerChecker(AllowAnonymousAccess())
        sScheduler.setServiceParent(s)
        return webPortal
    wp = stor.transact(_)

    def makeDomains():
        cc = ICredentialsChecker(stor)
        try:
            topDomain = cc.getDomain()
        except KeyError:
            # No default domain, so make one.
            cc.addDomain(config['domain'])

    stor.transact(makeDomains)

    def _():
        IPowerStation(stor).installPowerup(Sysup)
    stor.transact(_)

    for username, password, directories, mailboxes in config.users:
        a = stor.transact(makeUser, stor, username, password, POWERUPS + TAP_PUPS, False)

    def _():
        cp = IControlPanel(stor)
        cp.touch()
        cp.allowAction(IControlPanel.USER_CREATION, False)
        cp.allowAction(IControlPanel.TICKET_REDEMPTION, False)
    stor.transact(_)

    # Note: we need different portals because different protocols use different
    # checkers (as per Exarkun's request for different credentials for
    # different levels of security, even if they're all passwords)

    site = root.AtopSite(
        guard.SessionWrapper(
            guard.UsernamePasswordWrapper(wp)
        )
    )
    strports.service(config['webport'], site).setServiceParent(s)

    regadapt.AdapterRegistrationService().setServiceParent(s)

    stor.close()

    return s
