import time

from atop.store import Pool
from atop import powerup
from atop import credup
from atop import sched

from twisted.cred.checkers import ICredentialsChecker
from twisted.internet import defer
from twisted.python import components
from twisted.python import log

class IBackup(components.Interface):
    def nextPass(self):
        """Return the time to the next backup.
        """

    def queuedSubStores(self):
        """Return an iterator of the SubStores currently waiting for backup.
        """

class Backup(powerup.Storeup, sched.Schedulable):
    __implements__ = (powerup.Storeup.__implements__, sched.Schedulable.__implements__)

    interval = 60 * 60 * 1 # 24
    running = False

    def setUpPools(self, store):
        self.reset(1)
        bq = Pool(store, name="Backup Queue")
        self.queueRef = bq.referenceTo()
        store.setComponent(IBackup, self)

    def run(self):
        self.reset(self.interval)
        if self.running:
            return
        self.running = True

        self.fillQueue()
        self.store.transact(self.startBackup)

    def fillQueue(self):
        bq = self.queueRef.getItem()
        for a in ICredentialsChecker(self.store).getAvatars():
            if a not in bq:
                bq.addItem(a)

    def startBackup(self):
        q = self.queueRef.getItem()
        try:
            a = q.queryIndex().next()
        except StopIteration:
            self.running = False
        else:
            try:
                a.lock()
            except:
                from twisted.internet import reactor
                reactor.callLater(0, self.startBackup)
                return
            q.removeItem(a)
            d = self.backup(a)
            d.addErrback(self._ebBackup)
            d.addCallback(lambda r: self.store.transact(self.startBackup))

    def _backup(self, avatar):
        print 'Would have backed up', avatar.name

    def backup(self, avatar):
        # XXX Uncomment the following line to blow up the database
        # return avatar.transact(self._backup(avatar))
        return defer.succeed(avatar.transact(self._backup, avatar))

    def _ebBackup(self, failure):
        log.err(failure)
