from twisted.internet import task
from twisted.application import service, internet
from twisted.spread import pb
from twisted.cred.error import UnauthorizedLogin
from twisted.cred.portal import IRealm
from twisted.python.components import Interface
from twisted.python import log
from quotient.proto.twisted_sip import IContact
from atop import credup

import os,csv,time

SECRET = 'pai0Eiluzieni5Pu'

class CDRClient(pb.Referenceable):
    
    def __init__(self, store):
        self.store = store
        
    def remote_bill(self, account, time, destination):
        try:
            ava = credup.preauth(self.store, account)
            self.store.transact(IContact(ava).expend, time, destination)
        except UnauthorizedLogin:
            log.msg("Unknown CDR account code: " + account)

class CDRMonitor(pb.Avatar):

    def __init__(self, svc, client):
        self.service = svc
        self.client = client
        svc.clients.append(self.client)
    def logout(self):
        self.service.clients.remove(self.client)
        

class CDRMonitorService(service.Service):
    
    lastCDR = ()
    cdrTimeFmt = "%Y-%m-%d %H:%M:%S"
    cdrFilename = "/var/log/asterisk/cdr-csv/Master.csv"
    cdrFieldNames = "accountcode,src,dest,destContext,callerID,channel,destChannel,lastApp,lastData,startTime,endTime,duration,billableDuration,disposition,amaflags".split(',') 

    def billUntilCurrent(self):
        for cdr in csv.DictReader(self.cdrFile.readlines(),self.cdrFieldNames):
            if cdr['accountcode'] != "":
                for client in self.clients:                            
                    client.callRemote('bill', cdr['accountcode'], int(cdr['billableDuration']), cdr['dest'])
                self.lastCDR = (time.mktime(time.strptime(cdr['startTime'], self.cdrTimeFmt)), cdr)
                
    def startService(self):
        self.clients= []        
        if not os.path.exists(self.cdrFilename):
            raise RuntimeError, "the CDR file %s doesn't exist! Are you on drugs?" % self.cdrFilename
        self.cdrFile = open(self.cdrFilename)
        if self.lastCDR:
            for line in iter(self.cdrFile.readline,''):
                newcdr = iter(csv.DictReader((line,),self.cdrFieldNames)).next()
                oldtime, oldcdr = self.lastCDR
                newtime = time.mktime(time.strptime(newcdr['startTime'], self.cdrTimeFmt))
                if oldtime > newtime:
                    continue
                else:
                    if oldcdr == newcdr:
                        break
        self._billLoop = task.LoopingCall(self.billUntilCurrent)
        self._billLoop.start(10)

    def stopService(self):
        del self.clients
        del self._billLoop
        
class AsteriskRealm:
    __implements__ = IRealm

    def __init__(self, monitor):
        self.monitor = monitor

    def requestAvatar(self, avatarId, mind, *interfaces):
        if pb.IPerspective in interfaces:
            avatar = CDRMonitor(self.monitor, mind)
            return pb.IPerspective, avatar, avatar.logout
        else:
            raise NotImplementedError("no interface")

from twisted.cred.portal import Portal
from twisted.cred.checkers import InMemoryUsernamePasswordDatabaseDontUse
application = service.Application("cdrmonitor")
m = CDRMonitorService()
m.setServiceParent(application)
portal = Portal(AsteriskRealm(m))
checker = InMemoryUsernamePasswordDatabaseDontUse()
checker.addUser("asterisk", SECRET)
portal.registerChecker(checker)
internet.TCPServer(5039, pb.PBServerFactory(portal)).setServiceParent(application)
