# -*- test-case-name: quotient.test.test_garbage -*-
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

"""
Garbage collector!
"""

from atop import fwd
from atop import sched
from atop import store
from atop import powerup

import formless
from formless import webform

from twisted.python import log
from twisted.python import components

from quotient import cleanup
from quotient import mimemessage
from quotient.stats import formatSize
from quotient.proto.mboxlist import deferredPoolVisit

DAY = 60 * 60 * 24

class GarbageCounter:
    count = 0

class ICollector(components.Interface):
    pass

class GarbageCollector(powerup.Powerup, sched.Schedulable):

    __implements__ = (powerup.Powerup.__implements__, sched.Schedulable.__implements__, ICollector)

    dependencies = [sched.SubScheduler, cleanup.Cleanup]

    powerupName = "Trash Man"

    interval = DAY
    age = 7

    trash = store.referenced
    spam = store.referenced

    persistenceVersion = 2

    def upgradeToVersion2(self):
        self.touch()
        self.trash = self.trashRef.getItem()
        del self.trashRef
        self.spam = self.spamRef.getItem()
        del self.spamRef

    def upgradeToVersion1(self):
        self.touch()

        from quotient.queue import upgradeIndex

        t = self.trashRef.getItem()
        t.touch()

        t.__class__ = mimemessage.EmailPool
        t.suggestIndex('received', backwards=True)

        for iname in 'imapname', 'subject', 'sender':
            t.addIndex(store.NameIndex(iname))
            upgradeIndex(t, iname)


    def setUpPools(self, avatar):
        self.trash = mimemessage.EmailPool(avatar, name="Trash")
        avatar.getRootPool().addItem(self.trash)
        self.spam = cleanup.IClean(avatar).spamPool
        avatar.setComponent(ICollector, self)
        self.reset(self.interval)

    def _poolSize(self, pool):
        q = pool.queryIndex('size', result=store.INDEX)
        return sum(q)

    def run(self):
        self.reset(self.interval)
        return

        keep = not cleanup.ISpambayes(self.store).incinerateSpam
        then = sched.IScheduler(self.store).clock() - (self.age * DAY)

        s = self.spam
        t = self.trash
        q = s.queryIndex(store.RECEIVED, startKey=then, backwards=True)

        def collect(item):
            s.removeItem(item)
            if keep:
                t.addItem(item)

        return deferredPoolVisit(self.store.store, q, collect, 10
            ).addCallback(lambda r: self.reset(self.interval)
            ).addErrback(log.err
            )

    def empty(self):
        t = self.trash

        c = GarbageCounter()
        s = self._poolSize(t)
        q = self.trash.queryIndex()

        def remove(item):
            # XXX
            # Delete from database
            # Delete from disk
            c.count += 1
            t.removeItem(item)

        return deferredPoolVisit(self.store.store, q, remove, 10
            ).addCallback(lambda r: (c.count, s)
            )


from quotient.web import root
from quotient.taglib import page
from quotient.prefup import touchNForward
from quotient.stats import plural

class IGarbageCollector(formless.TypedInterface):
    age = formless.Integer(label="Collection Threshold",
                           description="Age of messages to move from Spam to Trash, in days.",
                           default="7")

    def emptyAll(self):
        """Permanently delete all message you have in your 'trash' folder."""
    emptyAll = formless.autocallable(emptyAll)

class GarbageDisplay(root.Base):
    __implements__ = (IGarbageCollector, root.Base.__implements__)

    document = page(title="Trash Manager Configuration")[ webform.renderForms() ]

    deleteMsgFmt = "Deleted %(count)s trash message%(plural)s totalling %(format)s."

    def deleteMessage(self, (c, s)):
        return self.deleteMsgFmt % {'count': c,
                                    'plural': plural(c),
                                    'format': formatSize(s)}

    def emptyAll(self):
        return self.original.empty(
            ).addCallback(self.deleteMessage
            )

    age = touchNForward("age")

    # XXX TODO Something about getting rid of the necessity for this
    postLocation = property(lambda self: self.original.storeID)
