
from nevow import tags

from twisted.web.resource import IResource, Resource
from atop.store import Pool, referenced, Store, Item, ItemReference
from twisted.cred.checkers import ICredentialsChecker
from atop.powerup import Powerup
from quotient import taglib
from quotient.web.root import Base
from twisted.python.components import Interface
from twisted.web.util import Redirect

class IPublic(Interface):
    """Public powerup marker.
    """

class PublicPowerup(Powerup):
    __implements__ = IPublic, Powerup.__implements__

    def setUpPools(self, av):
        av.setComponent(IPublic, self)
        self.publicPools = {}

    def makePoolPublic(self, pool):
        self.publicPools[pool.name] = pool.referenceTo()

class PublicPage(Base):
    def render_pools(self, context, data):
        return tags.ul[[tags.li[taglib.href(v)[k]]
                        for k,v in self.original.publicPools.items() if len(list(v.getItem().queryIndex(limit=1)))]]

    document = taglib.page("Public Page")[
        "Here are the pools made public by this user:",
        render_pools
        ]

    def render(self, request):
        self.remember(PublicLinker(getattr(request.getSession().getLoggedInRoot(),'store',None),
                                   self.original),
                      taglib.ILinkGenerator)
        return Base.render(self,request)

    def getChild(self, path, request):
        if not path:
            return self
        powerup = self.original
        pool = powerup.publicPools[path].getItem()
        return PublicPoolPage(powerup, pool)


class PublicPoolPage(Base):
    document = taglib.page('Public Pool Page')[
        "No index yet."
        ]
    def __init__(self, powerup, pool):
        Base.__init__(self, powerup)
        self.pool = pool

    def getChild(self, path, request):
        u = int(path)
        item = self.pool.getIndexItem('uid', u)
        # Now I need to convert the item to a resource which can generate
        # appropriate links: this means that the IResource adapter MUST be an
        # instance of Base (or something similar)
        rsrc = IResource(item)
        rsrc.remember(PublicLinker(getattr(request.getSession().getLoggedInRoot(),'store',None),
                                   self.original),
                      taglib.ILinkGenerator)
        return rsrc

def _domainFromHeader(store, request):
    """Retrieve host information and identify
    """
    domain = request.getHeader("host")
    if ':' in domain:
        domain = domain.split(':')[0]
    return domain

class UsersPage(Base):
    document = taglib.page("Users Directory",anonymous=True)[
        "Nothing to see here yet, sorry!"
        ]

    def getChild(self, path, request):
        """Renders children at http://divmod.com/users/foo/...
        """
        if not path:
            return self
        localpart = path
        origStore = self.original.store
        if not isinstance(origStore, Store):
            origStore = origStore.store
        domain = _domainFromHeader(origStore, request)
        cc = ICredentialsChecker(origStore)
        av = cc.getAvatar(localpart, domain)
        from quotient.public import IPublic
        pub = IPublic(av)
        return PublicPage(pub)

class PublicLinker:
    __implements__ = taglib.ILinkGenerator
    def __init__(self, viewingAvatar, publicPowerup):
        self.viewingAvatar = viewingAvatar
        self.publicPowerup = publicPowerup
        self.viewedAvatar = publicPowerup.store

    def urlTo(self, item, request):
        # if it's not an item, forget about it
        if not isinstance(item, (Item, ItemReference)):
            return

        if item.store == self.viewedAvatar:
            if isinstance(item, ItemReference):
                item = item.getItem()
            if hasattr(item,'name'):
                publicp = self.publicPowerup.publicPools.get(item.name)
                if publicp and publicp.storeID == item.storeID:
                    return '/users/%s/%s/0' % (self.viewedAvatar.name,
                                               item.name)
            for pool in item.iterpools():
                P = self.publicPowerup.publicPools.get(pool.name)
                if P and P.storeID == pool.storeID:
                    u = item.getIndexKey(pool, 'uid')
                    return '/users/%s/%s/%s' % (self.viewedAvatar.name,
                                                pool.name,
                                                u)

        if item.store == self.viewingAvatar:
            return '/me/%s' % item.storeID

