# -*- test-case-name: quotient.test.test_searchup -*-

# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import os
import time

from atop.powerup import Powerup, IPowerStation

from atop.tpython import Forgetter
from atop.store import IPipe, Pool, NameIndex
from atop import sched
from atop import credup

from lupy.indexer import Index

from twisted.python import log
from twisted.internet import threads, reactor
from twisted.internet import defer
from twisted.python.components import Interface, implements

from stats import Statoscope

from quotient import taglib
from quotient.web.root import Base
from quotient.prefup import touchNForward
from quotient.provisioner import ProvisionedPowerup, ETank, NotEnoughFuel, IProvisioner, IProvisionable
from quotient import sysup
from quotient.web import pools
from quotient.application import IFindApp, FindApp

from nevow import inevow
from nevow import tags
import formless 
from formless import webform

## XXX: I don't think this is necessary anymore,
## but didn't want to delete it until i confirmed
## -JS
class IBasicSearch(formless.TypedInterface):
    paused = formless.Boolean(
        label = "Pause indexing",
        description = 'Stops the indexer and queues new requests.'
        )


def hitRenderer(data, avatar):
    results = []
    for x in data:
        uid = x.get('uid')
        item = avatar.getItemByID(int(uid))
        try:
            summary = item['subject']
        except:
            summary = str(item)
        results.append(tags.p[
            tags.a(href=uid)[
                summary
            ]
        ])
    if not results:
        return tags.p["No results."]
    return results


class Hitter(list):
    def __init__(self, store, hits):
        self.store = store
        list.__init__(self, hits)

    def __iter__(self):
        for i in range(len(self)):
            yield self[i]

    def __getitem__(self, i):
        uid = int(list.__getitem__(self, i).get('uid'))
        return self.store.getItemByID(uid)


def hitRendererTable(hits, store, resource):
    if len(hits) == 0:
        return [tags.comment["We ain't found sh..."], tags.p["No matches found."]]
    resource.hitter = Hitter(store, hits)
    return webform.renderForms('searchResults')


class SearchDisplay(Base):
    __implements__ = Base.__implements__,

    def configurable_searchResults(self, ctx):
        rv = pools.PoolBrowser(self.hitter)
        ctx.remember(rv, inevow.IRendererFactory)
        return rv

    def render_search(self, context, data):
        request = context.locate(inevow.IRequest)
        if request.args.has_key('query'):
            query = request.args['query'][0]
            if query.startswith('/'):
                self.commandLine(context, request, query[1:])
                return ''
            charset = request.args['_charset_'][0]
            if charset:
                query = query.decode(charset)
            hits = self.original.search(query)
            return "Search results for: %s" % query.encode('utf8'), hits.addCallback(
                hitRendererTable, self.avatar, self
#                hitRenderer, self.avatar
            ).addErrback(
                log.err
            )
        else:
            return tags.p["Search:"], tags.form(id='search', action="")[
                tags.input(type="text", name="query"),
                tags.input(type="hidden",name="_charset_"),
                tags.input(type="submit")
            ]

    def commandLine(self, context, request, query):
        avatar = context.locate(credup.IAvatar)
        cmd, arg = query.split(' ', 1)
        cmd = cmd.strip().lower()
        argFirst = arg.split(' ', 1)[0].lower()
        if cmd in ('j', 'p'):
            if cmd == 'j':
               pool = avatar.getRootPool()
            elif cmd == 'p':
               pool = IPowerStation(avatar).getPowerupsPool()
            for p in pool.queryIndex('name', startKey=argFirst.capitalize()):
                if p.name.lower().startswith(argFirst):
                    request.redirect(str(p.storeID))
                    break
                else:
                    request.redirect(request.getHeader('referer'))
        else:
            ps = IPowerStation(self.original.store)
            for pup in ps.getPowerupsPool().queryIndex('name'):
                if hasattr(pup, 'command'):
                    pup.command(cmd, arg)
            request.redirect(request.getHeader('referer'))

    document = taglib.page(title="Search")[
        render_search,
    ]

    paused = touchNForward('paused')


class IIndexable(Interface):
    def indexableData(self):
        """
        Return a dictionary with strings as keys and values.
        For instance {'topic':'spam sandwich'} would index
        'spam sandwich' in a Lupy Field named 'topic'.
        See lupy.indexer.Index for more details.
        The searcher automatically adds the items storeID
        before saving it to the index. This allows you to
        retrieve your item from search results.
        """

class ISearch(Interface):
    def search(self, query):
        pass

def deferLater(when, f, *a, **kw):
    def later():
        defer.maybeDeferred(f, *a, **kw).chainDeferred(d)
    d = defer.Deferred()
    reactor.callLater(when, later)
    return d

INDEXING_INTERVAL = 0.1

class Searchup(ProvisionedPowerup, sched.Schedulable):
    __implements__ = sched.Schedulable.__implements__, IPipe, ISearch, IProvisionable, ProvisionedPowerup.__implements__,

    persistenceVersion = 1

    dependencies = [sched.SubScheduler, FindApp]
    storeDependencies = [sysup.ControlPanel]

    powerupName = 'Search'
    persistenceForgets = ['indexer', 'checkpointDefer', 'finishDefer']

    indexing = False
    paused = False

    checkpointDefer = None
    finishDefer = None

    tankID = ETank.INDEXING
    startCost = 1
    messageIndexCost = 1
    messageSearchCost = 1

    persistenceVersion = 3

    def upgradeToVersion3(self):
        """Cleanup upgrade.
        """
        self.touch()
        for attr in ['zindexes', 'pendingQueries', 'pendingDeletes', 'waitingForIndex']:
            if hasattr(self, attr):
                delattr(self, attr)

    def upgradeToVersion2(self):
        """Change my name to 'Search'
        """
        self.touch()
        self.setIndexKey('name','Search')

    def upgradeToVersion1(self):
        """Change the root of my indexing directory to be relative.
        """
        self.touch()
        base = self.store.store.filedir
        f = self.indexDir
        self.indexDir = base + '/' + f[f.find('/data/files/')+len('/data/files/'):]

    # TODO - Provisioning and Statoscope
    def setUpPools(self, avatar):
        ProvisionedPowerup.setUpPools(self, avatar)

        avatar.setComponent(ISearch, self)
        idxDir = avatar.newDirectory('indexer')
        create = not os.path.exists(os.path.join(idxDir, 'segments'))
        self.indexer = Index(idxDir, create=create)
        self.indexDir = idxDir
        self.indexer.setMergeFactor(25)
        self.scope = Statoscope()
        p = Pool(avatar, name='Indexing Queue')
        p.invisible = True
        self.indexQueuePoolRef = p.referenceTo()
        avatar.getRootPool().addItem(p)
        e = Pool(avatar, name='Indexing Errors')
        e.invisible = True
        self.indexErrorsPoolRef = e.referenceTo()
        avatar.getRootPool().addItem(e)

    def _keepAliveCallback(self, arg):
        """This callback exists to keep this object alive while the finishing
        deferred is in flight.
        """
        self.finishDefer = None
        return arg

    def finishedIndexing(self):
        """Return a deferred that will fire when the current indexing work has
        been completed.
        """
        if self.scheduled:
            self.finishDefer = defer.Deferred()
            self.finishDefer.addCallback(self._keepAliveCallback)
            return self.finishDefer
        else:
            return defer.succeed(None)

    def run(self):
        cp = sysup.IControlPanel(self.store.store)
        perm = sysup.IControlPanel.MESSAGE_INDEXING

        if self.paused or not cp.actionAllowed(perm):
            self.systemBlockedIndexing()
            return

        for item in self.indexQueuePoolRef.getItem().queryIndex():
            # Find and item that needs indexing and index it in a thread
            self._setCheckpointTimer()
            self._threadIndexItem(item)
            self.indexQueuePoolRef.getItem().removeItem(item)
            self.reset(INDEXING_INTERVAL)
            break
        else:
            if self.finishDefer:
                self.finishDefer.callback(None)

    def activate(self):
        self.indexer = Index(self.indexDir, create=False)
        self._setCheckpointTimer()

    def checkpoint(self):
        self.checkpointDefer = None
        self.indexer.close()

    def _setCheckpointTimer(self):
        if self.checkpointDefer:
            self.checkpointDefer.reset(15)
        else:
            self.checkpointDefer = reactor.callLater(15, self.checkpoint)

    def indexPool(self, pool):
        pool.touch()
        pool.addPipe(self.fwd(IPipe))

    def itemAdded(self, item):
        indexable = IIndexable(item, default=None)
        if indexable is not None:
            self.indexQueuePoolRef.getItem().addItem(item)
            self.reset(0)

    def _threadIndexItem(self, item, delete=False):
        """
        Index an item. Should only be called if we are
        not already in the process of indexing an item.
        """
        if delete:
            self.indexer.delete(uid=str(item.storeID))
            return item

        indexable = IIndexable(item, default=None)
        if indexable is None:
            return defer.succeed(item)
        try:
            kw = indexable.indexableData()
        except (LookupError, ValueError):
            self.indexErrorsPoolRef.getItem().addItem(item)
            kw = None
        if kw:
            kw['_uid'] = str(item.storeID)
            self.indexer.index(**kw)
        return item

    def systemBlockedIndexing(self):
        # System-wide block on indexing, try again later
        self.reset(60 * 30)

    def search(self, query):
        if not query:
            return defer.succeed([])
        else:
            result = self.indexer.find(query)
            return defer.succeed(result)

    def itemRemoved(self, item):
        indexable = IIndexable(item, default=None)
        if indexable is not None:
            self._setCheckpointTimer()
            self.indexer.delete(uid=str(item.storeID))

    def itemChanged(self, item, changes):
        pass

    # XXX cruft
    def _cbDoneIndexing(*a, **kw):
        pass
    def _ebIndexing(*a, **kw):
        pass
