# -*- test-case-name: quotient.test.test_extract -*-

# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from quotient.smarts.base import ExtractBase, ExtractCollectorBase
from twisted.python import components
from atop.store import IPipe

class AttachmentExtract(ExtractBase):
    def __init__(self, store, item, description='Unknown'):
        ExtractBase.__init__(self, store, item, description)


class IAttachmentPowerup(components.Interface):
    pass

# AttachmentCollector does not follow the .createExtract()
# pattern because determining whether an attachment is a
# duplicate or not is expensive.
#
# Ideas for cheapening detection:
#    trie
#    examine mail headers (cc, bcc)
#    examine mime headers (md5 digest might be available already)
class AttachmentCollector(ExtractCollectorBase):
    __implements__ = (IAttachmentPowerup, ExtractCollectorBase.__implements__, IPipe)

    powerupName = "Attachment Collector"

    def setUpPools(self, store):
        ExtractCollectorBase.setUpPools(self, store, name='Attachments', type=None)

    def extractItem(self, item):
        i = 0
        for att in item.getAttachments():
            i += 1
            A = AttachmentExtract(self.store, item, att.getAttachmentName())
            self.foundExtract(item, A)
            self.extractPool().addItem(A)
            if i > self.maxExtractsPer:
                return
