# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import os
import os.path
opj = os.path.join
from cStringIO import StringIO

from atop.powerup import Powerup
from atop.store import IPipe, Pool, Item, NameIndex
from atop.fwd import ItemForwardingComponent

from quotient.mimemessage import ICleanPoint, EmailPowerup
from quotient.smarts.base import ExtractBase, ExtractCollectorBase

from twisted.python import log
from twisted.internet import threads, reactor
from twisted.python.components import Interface
from twisted.web import static

from quotient import taglib
from quotient.web.root import Base
from quotient import provisioner

from nevow import tags
import formless
from formless import webform
from nevow import inevow

from quotient.web.root import Base, TabularBase, column

from nevow.tags import *

class TabularImage(TabularBase):
    def render_name(self, context, data):
        if hasattr(data, 'greyedOut') and data.greyedOut:
            return data.name
        else:
            return taglib.href(data)[str(data.name)]
    column(render_name, True, 300)

    def render_thumbnail(self, context, data):
        u2 = context.locate(taglib.ILinkGenerator).urlTo(data, context.locate(inevow.IRequest))
        return taglib.href(data)(title=data.name, style="border: 0px")[
            img(src=u2+'/thumbsup:%s.png' % (data.partNum))
        ]
    column(render_thumbnail, True, 200)

    def render_in(self, context, data):
        message = data.itemRef.getItem()
        return taglib.href(message)["In: %s" % message['subject']]

    def render_from(self, context, data):
        person = data.creatorRef and data.creatorRef.getItem()
        if person is None:
            return "From: Unknown"
        return taglib.href(person)["From: %s" % person.name]

class ImageCollectorDisplay(Base):
    def render_workCount(self, context, data):
        return self.original.workCount

    document = taglib.page(title="Image Collector")[
        tags.p["Messages Thumbnailed: ", render_workCount],
    ]


class IThumbnail(Interface):
    pass


class IImage(Interface):
    """
    @iva (storeID)
    @ivar size (x, y)
    @ivar name
    @location name (cheat for now)
    """

class ImageDisplay(Base):
    document = taglib.page(
        title=["Image: ", tags.directive('name')]
    )[
        tags.directive('image'),
        tags.p[ tags.directive('name') ],
        tags.p[ tags.directive('size') ],
        tags.p[ tags.directive('in') ],
        tags.p[ tags.directive('from') ],
    ]

    def render_image(self, context, data):
        src = "%s/%s.png" % (self.original.storeID, self.original.partNum)
        return tags.img(src=src, style="width: %spx; height: %spx;" % self.original.size)

    def render_name(self, context, data):
        return self.original.name

    def render_size(self, context, data):
        return "Width, Height: %s, %s" % self.original.size

    def render_in(self, context, data):
        message = data.itemRef.getItem()
        return taglib.href(message)["In: %s" % message['subject']]

    def render_from(self, context, data):
        person = data.creatorRef and data.creatorRef.getItem()
        if person is None:
            return "From: Unknown"
        return taglib.href(person)["From: %s" % person.name]

    def getDynamicChild(self, name, request):
        if name.startswith('thumbsup:'):
            if not os.path.exists(self.original.thumbnailPath):
                return taglib.missingImage()
            return static.File(self.original.thumbnailPath)
        if name == '%s.png' % self.original.partNum:
            message = self.original.itemRef.getItem()
            p = message.getPartByID(self.original.partNum)
            return static.Data(p.get_payload(True),
                            p.get_type())


class Image(ExtractBase):
    __implements__ = IImage

    def __init__(self, store, item, name, part, size, tpath):
        ExtractBase.__init__(self, store, item, name, part)
        self.size = size
        self.thumbnailPath = tpath


MODULE_NOT_PRESENT = "bad"
pil = None

class ImageCollector(ExtractCollectorBase):
    __implements__ = IPipe, IThumbnail, Powerup.__implements__, \
                     provisioner.ProvisionedPowerup.__implements__, provisioner.IProvisionable

    persistenceVersion = 1

    dependencies = [EmailPowerup, provisioner.Provisioner]
    powerupName = 'Image Collector'
    totalExtracted = 0
    tankID = provisioner.ETank.EXTRACTION
    startCost = 1
    byteCost = 0

    mimeTypes = ['image/']

    persistenceVersion = 1

    def upgradeToVersion1(self):
        def _():
            self.touch()
            f = self.cacheDir
            base = self.store.store.filedir
            self.cacheDir = base + '/' + f[f.find('/data/files/')+len('/data/files'):]

    # TODO - Provisioning and Statoscope
    def setUpPools(self, avatar):
        ExtractCollectorBase.setUpPools(self, avatar, name='Gallery', type='image/')
        self.cacheDir = avatar.newDirectory('cache')
        avatar.setComponent(IThumbnail, self)
        self.extractPool().suggestView('Gallery')
        self.working = None
        self.workCount = 0
        self.invisible = True
        self.thumbSize = 150

    def _loadPIL(self):
        global pil
        if pil is None:
            try:
                from PIL import Image
                pil = Image
            except ImportError:
                pil = MODULE_NOT_PRESENT

    def getImagePool(self):
        return self.imagePoolRef.getItem()

    def thumbnailAndImage(self, item, payload, part, bumnail=None):
        """
        Thumbnail an image. Create an Image.
        bumnail is what image should be used for images
        that can not be thumbnailed.
        """
        itemDir = opj(self.cacheDir, str(item.storeID), 'thumbsup')
        if not os.path.isdir(itemDir):
            os.makedirs(itemDir)
        imagepath = os.path.join(itemDir, '%s.png' % part.mimeID)

        # Open the image
        # TODO - split this into two steps?
        # 1) Try to open the image and catch the exception
        # 2) Try to resize and catch the exception
        try:
            image = pil.open(StringIO(payload))
            # Calculate scale
            imageSize = image.size
            width, height = image.size
            biggest = max(width, height)
            scale = 1/(biggest/float(self.thumbSize))
            # Do the thumbnailing
            image.resize((int(width * scale), int(height * scale)), True).save(imagepath)
        except IOError:
            # This can happen when we have some wierdness in the thumbnailing
            # e.g. exceptions.IOError: cannot write mode CMYK as PNG
            # TODO - Make bumnail a "No Thumbnail" image vs. None
            imagePath = bumnail
            imageSize = (0,0) # unknown
        # Create an Image and add it to the Images Pool
        imageItem = Image(self.store, item, part.getAttachmentName(), part, imageSize, imagepath)
        #self.imagesPoolRef.getItem().addItem(imageItem)
        self.workCount += 1
        return imageItem

    def extractPart(self, item, payload, part):
        self._loadPIL()
        if pil == MODULE_NOT_PRESENT:
            return
        image = self.thumbnailAndImage(item, payload, part)
        self.foundExtract(item, image)
        self.extractPool().addItem(image)
