# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from atop.powerup import Powerup

from twisted.python.components import Interface

from quotient.web.root import Base
from quotient.web.pools import PoolBrowser
from quotient.mimemessage import EmailPool
from quotient.taglib import page

from nevow.tags import h1, h2, p


class ISumUp(Interface):
    pass

class Summary(Powerup):
    __implements__ = ISumUp, Powerup.__implements__
    powerupName = 'Summary'
    
class SummaryDisplay(Base):
    __implements__ = Base.__implements__
    
    def render_summary(self, ctx, data):
        from quotient.application import IMailApp
        mailapp = IMailApp(self.original.store, None)
        yield h2['Divmod Summary']
        if mailapp is None:
            yield p['No new items.']
        else:
            # TODO - no hard coding. make configurable
            allPools = list(mailapp.appRoot.queryIndex('name'))
            ml = mailapp.appRoot.queryIndex('name', startKey='Mailing Lists',
                                            endKey='Mailing Lists')
            ml = ml.next()
            allPools.extend(list(ml.queryIndex('name')))
            nzPools = []
            zPools = []
            for pool in allPools:
                if isinstance(pool, EmailPool) and pool.newItemCount() is 0:
                    zPools.append(pool)
                elif isinstance(pool, EmailPool):
                    nzPools.append(pool)
              
            yield p["What's new:"]      
            if nzPools:
                yield PoolBrowser(nzPools)
            else:
                yield p['No new items.']

            yield p["Unchanged:"]
            if zPools:
                yield PoolBrowser(zPools)
            else:
                yield p['Everything has changed!']

    document = page(title="Divmod Summary")[
        render_summary]
