# -*- test-case-name: quotient.test.test_tap -*-
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import os

from twisted.python import usage
from twisted.application import service

from quotient import deployment

class Options(usage.Options):
    optParameters = [['domain', '', 'localhost', 'Domain to run services on.'],
                     
                     ['webport','w', '10080', 'Port to run web server on.'],
                     ['pop3port','p', '10110', 'Port to run POP3 server on.'],
                     ['smtpport', 's', '10025', 'Port to run SMTP server on.'],
                     ['imap4port', 'i', '10143', 'Port to run IMAP server on.'],
                     
                     ['websport','', '10443', 'Port to run SSL web server on.'],
                     ['pop3sport','', '10995', 'Port to run SSL POP3 server on.'],
                     ['smtpsport', '', '10465', 'Port to run SSL SMTP server on.'],
                     ['imap4sport', '', '10993', 'Port to run SSL IMAP server on.'],
                     
                     ['sipport','S', '15060', "Port to run a SIP registrar service on."],
                     ['pbport', 'm', '8787', "Port to run PB server on."],
                     ['asteriskport','',None, "Port of local Asterisk server. (internal use only)"],
                     
                     ['datadir','d', 'QuotientService/db', 'Directory name to put database in.'],
                     ['msgcal', 'm', 'QuotientService/files', 'Directory to put original email messages in.'],
                     
                     ['manhole-user', '', 'admin', 'Username for manhole service'],
                     ['manhole-pass', '', 'admin', 'Password for manhole service'],
                     ['manhole-port', '', '19293', 'Port for manhole service'],
                     
                     ['certificate', '', 'server.pem', 'Filename containing SSL certificate']
                     ]

    def postOptions(self):
        if not os.path.exists(self['certificate']):
            raise usage.UsageError("%r does not exist.  Specify a real file." % (self['certificate'],))

def makeService(config):
    return service.IService(deployment.deploy(
        dataDirectory = config['datadir'],
        fileDirectory = config['msgcal'],
        webLogFile = None,
        domainNames = [config['domain']],
        
        pop3Port = int(config['pop3port']),
        pop3sPort = int(config['pop3sport']),
        imap4Port = int(config['imap4port']),
        imap4sPort = int(config['imap4sport']),
        httpPort = int(config['webport']),
        httpsPort = int(config['websport']),
        smtpPort = int(config['smtpport']),
        smtpsPort = int(config['smtpsport']),
        sipPort = int(config['sipport']),
        pbPort = int(config['pbport']),
        
        manholeUser = config['manhole-user'],
        manholePassword = config['manhole-pass'],
        manholePort = config['manhole-port'],
        asteriskPort = int(config['asteriskport'] or 0),
        
        certificateFile = config['certificate'],
        privateKeyFile = config['certificate']))
