import atop
from atop import powerup

def getStore():
    import gc
    return [x for x in gc.get_referrers(atop.store.Store) if isinstance(x, atop.store.Store)][0]

def getAllAvatars():
    from atop.credup import ICredentialsChecker
    s = getStore()
    def _a():
        return [a for a in ICredentialsChecker(s).getAvatars()]
    return s.transact(_a)

def createUser(u, p, domain, ticketed, admin=True):
    from quotient.scripts import usercreate
    def d(**kw):
        return kw
    opts = d(username=u, password=p, domain=domain, ticketed=ticketed, admin=admin)
    st = getStore()
    st.transact(usercreate.createUser, st, opts)


def makeAdminUser(username, pwd): 
    """creates a new admin-level user
    @param username: self-explanatory
    @param pwd: password
    @returns: a Util object of the newly-created avatar
    """
    from quotient import sysup, puplist
    store = getStore()
    a = store.transact(sysup.makeUser, store, username, pwd, puplist.POWERUPS, free=True)
    u = Util(a.name)
    u.installPowerup(sysup.Adminup)
    return u

class Util(object):
    tu = None
    _avatarName, _avatar, _store = None, None, None

    def __init__(self, avatarName=None):
        self.avatarName = avatarName
        
    def _getStore(self):
        if not self._store:
            self._store = getStore()
        return self._store
    store = property(_getStore)

    def _getAvatarName(self):
        return self._avatarName

    def _setAvatarName(self, name):
        """note, this won't change self.avatar.name, it just changes what avatar we're dealing with"""
        if name != self._avatarName:
            self._avatar = atop.credup.preauth(self.store, name)    
            self._avatarName = name
    avatarName = property(_getAvatarName, _setAvatarName)

    def _getAvatar(self):
        return self._avatar

    def _setAvatar(self, a):
        if a != self._avatar:
            self._avatar = a
            self._avatarName = a.name
    avatar = property(_getAvatar, _setAvatar)
            
    def getPupPool(self):
        return self.store.powerstation.getPowerupsPool()

    def installTraceup(self):
        if self.installedTraceup:
            return

        from quotient.traceup import Traceup
        from atop.powerup import IPowerStation
        self.store.transact(IPowerStation(self.store).installPowerup, Traceup)
        self.tu = self.getTraceup()

    def printInstPups(self):
        for k in self.store.powerstation.powerups.iterkeys():
            print k

    def getTraceup(self):
        return self.store.powerstation.powerups['quotient.traceup.Traceup'].getItem()

    def installPowerup(self, pup):
        return self.store.transact(powerup.IPowerStation(self.avatar).installPowerup, pup)

    def printAvaPups(self):
        """prints an avatar's installed powerups"""
        for k in powerup.IPowerStation(self.avatar).powerups.iterkeys():
            print k

    def getPups(self):
        for v in powerup.IPowerStation(self.avatar).powerups.itervalues():
            yield v.getItem() 

    def getPowerup(self, klass):
        pup = None
        for p in self.getPups():
            if isinstance(p, klass):
                pup = p
        return pup

    def indexMessages(self):
        from quotient.application import IMailApp, MailAppRootPool
        from quotient.searchup import Searchup

        ma = IMailApp(self.avatar)
        rp = [p for p in ma.getApplicationNav() if isinstance(p, MailAppRootPool)][0]
        sup = self.getPowerup(Searchup)
        
        def foo():
            pool = [p for p in rp.queryIndex()][0]
            for message in pool.queryIndex():
                sup.itemAdded(message)
        self.store.transact(foo)
        

class MailUtil(Util):
    _mailapp = None
    def _getMailApp(self):
        if not self._mailapp:
            from quotient.application import IMailApp
            self._mailapp = IMailApp(self.avatar)
        return self._mailapp
    mailapp = property(_getMailApp)
    
    def loadMailDir(self, path=None):
        if path == None:
            from quotient import testdata
            from twisted.python import util
            path = util.sibpath(testdata.__file__, '')

        from quotient.grabbers import IGrabberAdder
        IGrabberAdder(self.avatar).importDirectory(path)
