
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from atop.test import unittest as atoptest
from atop import sched
from atop import store

from twisted.trial import unittest

from quotient.test import helper
from quotient.smarts import question
from quotient.smarts import url

class StringMIMEPart:
    mimeID = 7
    def __init__(self, s):
        self.s = s
    def get_payload(self, *a, **kw):
        return self.s

class Extract(atoptest.TestCase, helper.ProvisionerMixin):
    storePowerups = [sched.SystemScheduler]

    def dbTearDown(self):
        # nc = sched.IScheduler(self.store).nextCall
        # nc and nc.cancel()
        pass

    def dbtestExtractionList(self):
        a = self.createAvatar([question.QuestionCollector])
        c = question.IQuestionPowerup(a)
        p = c.extractRef.getItem()

        E = question.QuestionExtract

        i1 = store.Item(a)
        e1 = [E(a, i1, 1, 1, 2), E(a, i1, 2, 3, 4), E(a, i1, 2, 5, 6)]

        i2 = store.Item(a)
        e2 = [E(a, i2, 2, 7, 8), E(a, i2, 2, 9, 10)]

        for i in e1 + e2:
            p.addItem(i)

        self.assertEquals(list(c.queryExtracts(i1)), e1)
        self.assertEquals(list(c.queryExtracts(i2)), e2)
    dbtestExtractionList.skip = 'Needs to use a MIMEMessage for testing, not an Item'


class Question(unittest.TestCase):
    def testQuestions(self):
        simple = [("How are you?", ["How are you?"]),
                  ("It is nice here.  How is it there?", ["How is it there?"]),
                  ("The server broke.  Please fix it?", ["Please fix it?"])]
        complex = []
        #    ("The server broke... please fix it?",
        #     ["The server broke... please fix it?"])]
        multiple = [
            ("What does it mean?  Where did it come from?",
             ["What does it mean?", "Where did it come from?"]),
            ("How do I do this?  How do I do that?",
             ["How do I do this?", "How do I do that?"])]

        for (i, o) in simple + complex + multiple:
            self.assertEquals([i[b:e] for (b, e) in question.questionsFromText(i)], o)

class PhoneMatching(unittest.TestCase):
    def testNorthAmerican(self):
        simple = [("123-4567", [("", "123", "4567")]),
                  ("123-456-7890", [("123", "456", "7890")]),
                  ("098.765.4321", [("098", "765", "4321")]),
                  ("567 890 1234", [("567", "890", "1234")])]

        complex = [("leading text 555-333-8888 trailing", [("555", "333", "8888")]),
                   ("213 555-653-3567 more numbers!", [("555", "653", "3567")])]

        multiple = [("123-456-7890 980-765-4321",
                     [("123", "456", "7890"), ("980", "765", "4321")]),
                    ("leading 123-456-7890 middle 123-456-5432 done",
                     [("123", "456", "7890"), ("123", "456", "5432")])]

        from quotient.smarts.phone import NorthAmericanPhoneNumber as pnre
        for (i, o) in simple + complex + multiple:
            self.assertEquals(pnre.findall(i), o)

class URLMatching(atoptest.TestCase, helper.ProvisionerMixin):
    storePowerups = [sched.SystemScheduler]

    def dbSetUp(self):
        self.avatar = self.createAvatar([url.URLCollector])
        self.url = url.IURLPowerup(self.avatar)

    def dbTearDown(self):
        # nc = sched.IScheduler(self.store).nextCall
        # nc and nc.cancel()
        pass

    def dbtestItAll(self):
        part = StringMIMEPart("vrml://www.hostname.tld/fizzpop oaiwjdoda https://host.tld/fizz\n"
                              "wdopkqpwdk ftp://host.tld/fizz 39i sip://www.host.tld/fizzpop\n"
                              "http://host.tld/fizz http://localhost/fizzpop edk9\n"
                              "junk://www.host.tld/fizzpop http://127.0.0.1/fizzpop\n")
        msg = store.Item(self.avatar)
        self.url.extractPart(msg, part)

        expected = ["https://host.tld/fizz", "ftp://host.tld/fizz", "http://host.tld/fizz"]

        for extract in self.url.queryExtracts(msg):
            begin, end = extract.begin, extract.end
            self.assertEquals(expected.pop(0), part.s[begin:end])
    dbtestItAll.skip = 'Needs to use a MIMEMessage for testing, not an Item'
