# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import time

from atop import sched
from atop import store
from atop.test import unittest

from quotient.test.emaildata import test_messages
from quotient.test import helper

from quotient.mimemessage import messageFromStructure
from quotient.gcup import GarbageCollector, ICollector, DAY

class GarbageCollection(unittest.TestCase, helper.ProvisionerMixin):
    storePowerups = [sched.SystemScheduler]

    def dbSetUp(self):
        self.avatar = self.createAvatar([GarbageCollector])

    def dbTearDown(self):
        dc = sched.IScheduler(self.store).nextCall
        dc and dc.cancel()

    def dbtestTimeThreshold(self):
        c = ICollector(self.avatar)

        spam = c.spam
        trash = c.trash

        now = sched.IScheduler(self.avatar).clock()

        for i in range(-14, 0):
            when = now + (i * DAY) + 60
            m = messageFromStructure(self.avatar, [], "")
            m.setReceived(time.gmtime(when))
            spam.addItem(m)

        self.assertEquals(len(list(spam.queryIndex())), 14)
        self.assertEquals(len(list(trash.queryIndex())), 0)

        unittest.deferredResult(c.run())

        self.assertEquals(len(list(spam.queryIndex())), 7)
        self.assertEquals(len(list(trash.queryIndex())), 7)

        unittest.deferredResult(c.empty())

        self.assertEquals(len(list(spam.queryIndex())), 7)
        self.assertEquals(len(list(trash.queryIndex())), 0)

        m = spam.queryIndex().next()
        spam.removeItem(m)
        trash.addItem(m)

        unittest.deferredResult(c.empty())

        self.assertEquals(len(list(spam.queryIndex())), 6)
        self.assertEquals(len(list(trash.queryIndex())), 0)

    todo = "Figure out where all the locks are going"
