# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

"""
"""

from atop.test import unittest

from quotient.puplist import ALL
from quotient import mimebakery
from quotient.test import helper
from quotient import mimemessage
from quotient import cleanup
from quotient import searchup
from quotient import voiceup
from quotient.proto import imapup

class IMAPListener:
    pass

class MessageSet(list):
    pass

class IMAPInteraction(unittest.TestCase, helper.ProvisionerMixin):
    def dbSetUp(self):
        self.avatar = self.createAvatar(ALL)

        msg = mimebakery.Recipe(self.avatar, "recipient")
        msg.text("Here is message body")

        arrival = mimemessage.IMIMEDelivery(self.avatar).getArrivalRef().getItem()

        for i in range(15):
            arrival.addItem(msg.bake())

    def dbTearDown(self):
        nc = searchup.ISearch(self.avatar).checkpointDefer
        nc and nc.cancel()

    def dbtestFlagChanges(self):
        clean = cleanup.IClean(self.avatar).cleanPool
        P1 = unittest.deferredResult(imapup.deferredOpenPool(self.avatar, clean, False))
        P2 = unittest.deferredResult(imapup.deferredOpenPool(self.avatar, clean, False))

        f1List = []
        f2List = []
        L1 = IMAPListener()
        L2 = IMAPListener()

        L1.flagsChanged = f1List.append
        L2.flagsChanged = f2List.append

        P1.addListener(L1)
        P2.addListener(L2)


        for msg in clean.queryIndex():
            msg.changeFlags(None, mimemessage.MIMEMessage.SEEN)

            self.assertEquals(f1List, [{P1.uidByItem(msg): ['\\Seen']}])
            self.assertEquals(f2List, [{P1.uidByItem(msg): ['\\Seen']}])

            del f1List[:]
            del f2List[:]

    def dbtestMovingMessages(self):
        clean = cleanup.IClean(self.avatar).cleanPool
        P1 = imapup.deferredOpenPool(self.avatar, clean, False)
        P1 = unittest.deferredResult(P1)

        f = clean.queryIndex().next

        msg = f()
        uid = P1.uidByItem(msg)
        clean.removeItem(msg)

        # Try to get the message we just removed
        # This should succeed, because we haven't
        # told the client that it is gone, yet.

        mset = MessageSet([uid])
        self.assertEquals([x[1].msg for x in P1.fetch(mset, True)], [msg])

        mset = MessageSet([1])
        self.assertEquals([x[1].msg for x in P1.fetch(mset, False)], [msg])
