
from atop.test import unittest

from quotient import tasks
from quotient import searchup

from quotient.test import helper

def pLen(pool):
    return len(list(pool.queryIndex()))

class GoalTestCase(unittest.TestCase, helper.ProvisionerMixin):

    def dbSetUp(self):
        self.avatar = self.createAvatar([tasks.Taskmaster])

    def dbTearDown(self):
        nc = searchup.ISearch(self.avatar).checkpointDefer
        nc and nc.cancel()
        
    def dbtestBasic(self):
        tm = tasks.ITaskmaster(self.avatar)
        self.assertEquals(pLen(tm.getGoalsPool()), 1)
        self.assertEquals(pLen(tm.getOwnersPool()), 1)
        
    def dbtestGoals(self):
        tm = tasks.ITaskmaster(self.avatar)
        g = tm.newGoal("A Goal")
        p = g.getTasksPool()

        self.assertEquals(g.name, "A Goal")
        g.editGoal("Not really a goal, just a test")
        self.assertEquals(g.name, "Not really a goal, just a test")

        self.assertEquals(pLen(tm.getGoalsPool()), 2)
        
        t = g.newTask("Fake task")
        self.failUnless(t in p)
        g.removeTask(t)
        self.failIf(t in p)

        t = g.newTask("Fake task 2")
        self.failUnless(t in p)
        tm.removeTask(t)
        self.failIf(t in p)

        #self.assertRaises(ValueError, g.removeTask(t))

        t = g.newTask("Fake task 3")
        self.failUnless(t in p)
        newT = t.split()
        itext = newT.indexableData()['text']
        self.assertEquals(itext, "Split from Fake task 3")
        self.assertEquals(str(newT), itext)
        tm.removeTask(newT)
        self.failIf(newT in p)
        
    def dbtestOwners(self):
        tm = tasks.ITaskmaster(self.avatar)
        sam = tm.newOwner('Sam')
        jo = tm.newOwner('Jo')
        op = tm.getOwnersPool()
        self.assertEquals(pLen(op), 3)
        t = tm.newTask('Bake a cake')
        self.assertEquals(t.ownerRef.getItem(), tm.nobodyRef.getItem())
        sam.ownTask(t)
        self.assertEquals(t.ownerRef.getItem(), sam)
        jo.ownTask(t)
        self.assertEquals(t.ownerRef.getItem(), jo)

    def dbtestTasks(self):
        tm = tasks.ITaskmaster(self.avatar)
        task = tm.newTask('Bake 3 cakes')
        self.assertEquals(task.goalRef.getItem(), tm.noGoalRef.getItem())
        self.assertEquals(pLen(tm.getTasksPool()), 1)
        task = tm.newTask('Put icing on the cake')
        ng = tm.noGoalRef.getItem()
        self.assertEquals(pLen(ng.getTasksPool()), 2)
        task.setOriginalEstimate(5)
        self.assertEquals(task.remain, 5)        
        task.editTask(5, 'Icing and cherry on the top', 2, 0, 0)
        self.assertEquals(task.remain, 3)   
