import os

from atop.test import unittest
from atop import credup
from atop import sched

from quotient.scripts import usercreate
from quotient import sysup
from quotient import mimemessage
from quotient import grabbers

class UtilTestCase(unittest.TestCase):
    storePowerups = [sched.SystemScheduler, 
                     credup.StoreRealm, 
                     sysup.ControlPanel]

    def dbSetUp(self):
        cc = credup.ICredentialsChecker(self.store)
        cc.addDomain("testdom")
        cc.addDomain("trickdom")
    
    def makeUser(self, admin):
        opts = {'username': 'testuser',
                'password': 'testpass',
                'domain': 'testdom',
                'ticketed': True,
                'admin': admin}
        return usercreate.createUser(self.store, opts)

    def dbtestCreateUser(self):
        ava = self.makeUser(False)

        self.assertEquals(ava.name, 'testuser')
        self.assertEquals(ava.getPassword(True), 'testpass')
        self.assertEquals(ava.getPassword(False), 'testpass')

        cc = credup.ICredentialsChecker(self.store)

        self.assertIn('testdom', [d.name for d in cc.getDomains()])
        self.assertEquals(list(cc.getAvatars()), [ava])

        dom = cc.getDomain("testdom")
        self.assertIdentical(dom.getIndexItem('name', 'testuser'), ava)

    def dbtestCreateAdmin(self):
        ava = self.makeUser(True)
        admin = sysup.ISystemAdministrator(ava)

    def dbtestImportDirectory(self):
        # Make a directory full of mail
        maild = self.mktemp()
        os.mkdir(maild)
        for i in range(5):
            mailf = file(os.path.join(maild, str(i)), 'w')
            mailf.write("Header: value\n\nBody!\n")
            mailf.flush()
            mailf.close()

        ava = self.makeUser(False)
        usercreate.importDirectories(ava, {'directories': [maild]})
        arrival = mimemessage.IMIMEDelivery(ava).getArrivalRef().getItem()
        self.assertEquals(len(list(arrival.queryIndex())), 5)

    def dbtestImportMailbox(self):
        # Make an mbox full of messages
        mailf = self.mktemp()
        fObj = file(mailf, 'w')
        for i in range(5):
            fObj.write('From Johnny Q. Public Tue Dec 12 12:23:34 2000\n')
            fObj.write('Header: value\n\n')
            fObj.write('Tra la la la la \n')
        fObj.close()

        ava = self.makeUser(False)
        usercreate.importMailboxes(ava, {'mailboxes': [mailf]})
        arrival = mimemessage.IMIMEDelivery(ava).getArrivalRef().getItem()
        self.assertEquals(len(list(arrival.queryIndex())), 5)

    def dbtestAddPOP3Grabbers(self):
        ava = self.makeUser(False)
        usercreate.addPOP3Grabbers(ava, {'pop3gs': ['u:p:h:1', 'U:P:H:2']})

        g = grabbers.IGrabberAdder(ava).grabbers
        self.failUnless(isinstance(g[0], grabbers.POP3GrabberFactory))
        self.assertEquals(g[0].username, 'u')
        self.assertEquals(g[0].password, 'p')
        self.assertEquals(g[0].host, 'h')
        self.assertEquals(g[0].port, 1)

        self.failUnless(isinstance(g[1], grabbers.POP3GrabberFactory))
        self.assertEquals(g[1].username, 'U')
        self.assertEquals(g[1].password, 'P')
        self.assertEquals(g[1].host, 'H')
        self.assertEquals(g[1].port, 2)

    def dbtestaddIMAP4Grabbers(self):
        ava = self.makeUser(False)
        usercreate.addIMAP4Grabbers(ava, {'imap4gs': ['u:p:h:1', 'U:P:H:2']})

        g = grabbers.IGrabberAdder(ava).grabbers
        self.failUnless(isinstance(g[0], grabbers.IMAP4GrabberFactory))
        self.assertEquals(g[0].username, 'u')
        self.assertEquals(g[0].password, 'p')
        self.assertEquals(g[0].host, 'h')
        self.assertEquals(g[0].port, 1)

        self.failUnless(isinstance(g[1], grabbers.IMAP4GrabberFactory))
        self.assertEquals(g[1].username, 'U')
        self.assertEquals(g[1].password, 'P')
        self.assertEquals(g[1].host, 'H')
        self.assertEquals(g[1].port, 2)
