# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.
#

import time

from nevow import context
from nevow.test import test_flatstan

from twisted.web import microdom
from twisted.trial import unittest

from quotient.web import root
from quotient.web import scrubber

from quotient.test import emaildata

class BenchmarkPages(test_flatstan.Base):
    nRenders = 1000

    def benchmarkPrecompile(self):
        docs = [root.AnonymousRoot, root.UserFrontPage, root.NotRenderable]
        count = self.nRenders * len(docs)

        startWall = time.time()
        startCPU = time.clock()
        for i in range(self.nRenders):
            for docContainer in docs:
                self.render(docContainer.document, data="", precompile=True)
        endWall = time.time()
        endCPU = time.clock()

        totWall = endWall - startWall
        totCPU = endCPU - startCPU
        
        self.recordStat({'CPU': ((count / totCPU), "Precompiles_per_cpu_sec"),
                         'Wallclock': ((count / totWall), "Precompiles_per_wall_sec")})


class Scrubber(unittest.TestCase):
    def testAtLeastRun(self):
        for msg in emaildata.html_messages:
            dom = microdom.parseString(msg, beExtremelyLenient=True)
            scrubber.scrub(dom)
    testAtLeastRun.skip = "This totally locks right now"


