# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import rfc822

from twisted.python import log

from quotient.proto import smtpout
from quotient import mimemessage
from quotient import taglib

from formless import webform
from nevow import formless
from nevow import inevow
from nevow import stan

from quotient.mimemessage import MIMEMessage, headerParams
from quotient.grabbers import IGrabberAdder
from twisted.cred.checkers import ICredentialsChecker
from quotient import prefup
from quotient.web import root
from quotient.taglib import *
from nevow.tags import *
from nevow import inevow
from formless import iformless


class EmailAddress(formless.Typed):
    def coerce(self, val):
        try:
            a = list(rfc822.AddressList(val))
        except:
            log.err()
            raise formless.InputError("Please enter an email address")
        else:
            if len(a) != 1:
                raise formless.InputError("Please enter an email address")
            # XXX - This is not a long term requirement
            if '@' not in a[0][1]:
                raise formless.InputError("Please include a hostname in the email address")
            return val

class EmailAddressList(formless.Typed):
    requiredFailMessage = "Please enter an email address."
    def coerce(self, val):
        try:
            a = list(rfc822.AddressList(val))
        except:
            log.err()
            raise formless.InputError("Please enter one or more email addresses")
        else:
            for e in a:
                # XXX - This is not a long term requirement
                if '@' not in e[1]:
                    raise formless.InputError("Please include a hostname in the email address")
            return val
        return ''

class IComposer(formless.TypedInterface):
    def sendMail(self,
                 request=formless.Request(),
                 to=EmailAddressList(required=True, label="To:"),
                 cc=EmailAddressList(label="CC:"),
                 bcc=formless.String(label="BCC:"),
                 ## replyTo=EmailAddress(label="Reply to:"),
                 fromAddress=formless.Choice(label="From:", choicesAttribute="fromAddresses"),
                 subj=formless.String(label="Subject:"),
                 text=formless.Text(label="Body:"),
                 attachment=formless.FileUpload(label="Attachment:"),
                 action=formless.String(label="Forwarding:", hidden=True, default=0),
                 inReplyTo=formless.String(label="In-Reply-To:", hidden=True, default=0),
                 submit=formless.Button(label="Send mail")):
        """
        Send Mail to someone.
        """
        return None
    sendMail = formless.autocallable(sendMail, action="Send", revealed=True)


def reSubject(m, pfx='Re: '):
    newsubject = m['subject'] or ''
    if not newsubject.lower().startswith(pfx.lower()):
        newsubject = pfx + newsubject
    return newsubject


def quoteBody(m):
    try:
        dispart = m.getDisplayPart()
    except ValueError:
        return ''
    else:
        newtext = dispart.get_payload().split('\n')
        return [ '\n> '.join(newtext) ]



def aspectProperty(aspect, name):
    return property(
        lambda self: getattr(aspect(self.avatar), name),
        lambda self, val: setattr(aspect(self.avatar), name, val)
    )


def hack(it):
    def kcah(ctx, data):
        R = it(ctx, data)
        if callable(R): return ''
        return R
    return kcah


from nevow.blocks import block, line, collapser

def remove(ctx, data):
    #import pdb; pdb.set_trace()
    return ''


class Compose(root.Base):
    adaptsAvatar = False

    __implements__ = IComposer, root.Base.__implements__

    def render_attachments(self, context, data):
        forwarding = context.locate(iformless.IFormDefaults).getDefault('sendMail.forwarding')
        if forwarding:
            fwdmsg = self.original.store.getItemByID(int(forwarding))
            assert isinstance(fwdmsg, MIMEMessage)
            attachmentNames = [fwdpart.get_filename('Attachment') for fwdpart in fwdmsg.getAttachments()]
            if not attachmentNames:
                return ''
            return ['Attachments:',
                [ div[ name ] for name in attachmentNames ],
                br
            ]
        return ''

    def checkinput(self, ctx, data):
        to = ctx.arg('newMessageTo')
        if to is not None:
            from formless import iformless
            ctx.locate(iformless.IFormDefaults).getAllDefaults('sendMail')['to'] = to

        for act in ['reply', 'forward', 'redirect']:
            msgnum = ctx.arg(act)
            if msgnum:
                msg = self.original.store.getItemByID(int(msgnum))
                getattr(self, 'do_%s' % act, lambda req, msg: None)(ctx, msg)
        return ''

    document = taglib.page("Compose email")[
        checkinput,
        webform.renderForms()[
            form(
                pattern='freeform-form', id=slot('form-id'), 
                action=slot('form-action'), method='POST',
                _class='compose-form'
            )[
                block(width="120px", height="32px", vertical_align='top')[
                    input(type="image", src="/images/submit-buttons/compose.png", style="width: 101px; height: 32px")],
                block(width="700px")[
                    span(style="color: red")[ slot('form-error') ],
                    invisible(pattern='argument', render=remove)[
                        line()[
                            block(width="84px")[slot('label')], 
                            block(width="325px")[slot('input')],
                            span[
                                slot('error'), slot('description')]]], 
                    invisible(pattern='argument!!to', render=remove)[
                        block(width="84px")[slot('label')], 
                        block(width="325px")[slot('input')],
                        span[
                            slot('error'), slot('description')]], 
                    slot('argument!!fromAddress'),
                    slot('argument!!to'),
                    collapser('More options...', 'Fewer options...', [slot('argument!!cc'), slot('argument!!bcc')]),
                    br,
                    slot('argument!!subj'),
                    slot('argument!!text')[
                        invisible(pattern='argument!!text')[
                            slot('input')]],
                    slot('argument!!attachment'),
                    render_attachments,
                    slot('argument!!request'), 
                    slot('argument!!action'),
                    slot('argument!!inReplyTo')]]]]

    def setDefaults(self, ctx, msg=None, flags = []):
        D = ctx.locate(iformless.IFormDefaults).getAllDefaults('sendMail')
        D["action"] = "%s,%s" % (','.join(['%s' % x for x in flags]), str(msg.storeID))
        return D

    def do_reply(self, ctx, msg):
        formContents = self.setDefaults(ctx, msg, flags=[MIMEMessage.ANSWERED])
        formContents['to'] = msg['reply-to'] or msg['from'] or ''
        formContents['subj'] = reSubject(msg)
        formContents['replyTo'] = msg.addressedTo or prefup.IPrefs(self.avatar).getPref('replyto') or ''
        newtext = quoteBody(msg)

        origdate, origfrom = msg['date'], msg['from']
        if not origdate:
            origdate = "an indeterminate time in the past"
        if not origfrom:
            origfrom = "someone who chose not to be identified"
        replyhead = 'On %s, %s wrote:\n>' % (origdate.strip(), origfrom.strip())
        formContents["text"] = replyhead + ('\n> '.join(newtext))
        formContents['inReplyTo'] = msg['message-id']


    def do_forward(self, ctx, msg):
        formContents = self.setDefaults(ctx, msg, flags=[MIMEMessage.FORWARDED])
        newsubject = reSubject(msg, 'Fwd: ')
        formContents["subj"] = newsubject
        formContents["replyTo"] = msg.addressedTo or prefup.IPrefs(self.avatar).getPref('replyto') or ''
        newtext = quoteBody(msg)

        reply = ["\nBegin forwarded message:\n"]
        reply.extend(['%s: %s' % (x, msg[x]) for x in ['From', 'Date', 'To', 'Subject', 'Reply-to'] if msg[x]])
        reply.append('')
        reply.extend(newtext)

        formContents["text"] = '\n> '.join(reply)

    def do_redirect(self, ctx, msg):
        formContents = self.setDefaults(ctx, msg, flags=[MIMEMessage.REDIRECTED])
        newsubject = reSubject(msg)
        formContents["subj"] = newsubject
        formContents["replyTo"] = msg['from'] or ''

        try:
            fullText = [msg.getDisplayPart().get_payload().splitlines()]
        except ValueError:
            fullText = ['']

        for i, x in enumerate(fullText):
            fullText[i] = '\n'.join(x)
        formContents["text"]='\n'.join(fullText)

    replyTo = ''

    def calculateFromAddresses(self):
        avatar = self.original.store
        store = avatar.store
        grabberaddresses = [g.address for g in IGrabberAdder(avatar).grabbers if g.address]
        selfaddresses = ["%s@%s" % (avatar.name, d.name) for d in ICredentialsChecker(store).getDomains() if d.getIndexItem('name', avatar.name, default=None) is avatar]
        return grabberaddresses + selfaddresses

    fromAddresses = property(calculateFromAddresses)

    def sendMail(self, request, to, cc, bcc, fromAddress, subj, text, attachment, action, submit, inReplyTo):
        # attachment is a FieldStorage instance, ewww
        # XXX there are NO unit tests for this method and I don't know how to
        # write them.
        if attachment is not None and attachment != '' and attachment.value:
            attachments = [
                (attachment.value,
                 attachment.headers.get('content-type') or 'application/octet-stream',
                 headerParams(attachment.headers['content-disposition'] or '')[1].get('filename') or 'Attachment'),
                ]
        else:
            attachments = []
        ## Are we forwarding, replying or redirecting a message?
        if action != '0' and action:
            print "ACTION", `action`
            work = [int(x) for x in action.split(',')]
            flags = work[:-1]
            messagenum = work[-1]
            msg = self.original.store.getItemByID(messagenum)
            assert isinstance(msg, MIMEMessage)
            msg.changeFlags(None, flags)
            if MIMEMessage.FORWARDED in flags:
                for fwdpart in msg.getAttachments():
                    attachments.append((
                        fwdpart.get_payload(decode=True),
                        fwdpart.inferType(),
                        fwdpart.get_filename('Attachment'),
                        ))
        om = smtpout.IOutgoingMail(self.original.store)
        om.sendString(messageText=text or '',
                      attachments=attachments,
                      to=to,
                      replyTo="",
                      fromAddress=fromAddress,
                      cc=cc,
                      bcc=bcc,
                      subject=subj or 'No Subject',
                      inReplyTo=inReplyTo)

