# -*- test-case-name: quotient.test.test_tdb -*-
# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.
##

import math, time
from itertools import islice, chain, repeat
from pprint import pformat

from urllib import urlencode

from atop import store

from twisted.python import reflect
from twisted.python import log

from quotient.web import root
from quotient import taglib
from quotient.taglib import getAttribute, eitherOr, name, anId, nameOrId, href, stickyhref
from quotient.mimemessage import EmailPool, MIMEMessage
from quotient import iquotient
from quotient.prefup import IPrefs
from quotient import application

from nevow import inevow
from nevow.inevow import ISerializable, IRendererFactory, IRequest, IResource
from formless import configurable
import formless
from nevow import compy

from nevow.tags import *
from nevow.url import here, gethere

from atop.store import INDEX, STORE_ID
from atop import credup

class MetaTabularAggregator(type):
    def metasetup(self, C):
        ifaces = compy.getInterfaces(C)
        ifaces.reverse()
        ifaces.append(C)
        for interface in ifaces:
            Ctab = compy.getAdapterFactory(interface, iquotient.ITabularRenderer, None)
            if Ctab is not None:
                self.addMethodsFrom(Ctab)

    def addMethodsFrom(self, klass):
        for k, v in klass.__dict__.iteritems():
            if k.split("_")[0] in ('base', 'action', 'render'):
                setattr(self, k, v)

_aggregators = {}

def TabularAggregator(o):
    C = o.__class__
    if _aggregators.has_key(C):
        return _aggregators[C](o)
    mta = MetaTabularAggregator("TabularAggregator_"+(reflect.qual(C).replace('.','_')), (root.TabularBase,), {})
    mta.metasetup(C)
    _aggregators[C] = mta
    return mta(o)

class TabularPool(root.TabularBase):
    # TODO - Move email specific code out of here
    def render_unread(self, context, data):
        if isinstance(data, EmailPool):
            return str(data.unreadCount())
        return ''
    root.column(render_unread, True, 60, "Unread")
    
    def render_new(self, context, data):
        if isinstance(data, EmailPool):
            return str(data.newItemCount())
        return ''
    root.column(render_new, True, 60, "New")

class TabularNothing(root.TabularBase):
    storeID = 0
    name = property(lambda self: "A %s" % self.original.__class__.__name__)


def kindIcon(context, data):
    q = reflect.qual(data.__class__)
    return img(src=q, title=q, alt=q), q, " "


def anIntrospector(context, data):
    return pformat(data.__dict__)


#CRAP = "float: left; width: %spx; height: 1.25em; overflow: hidden"
#CHECKBOX = "float: left; height: 1.25em; width: 25px; overflow: hidden"
CRAP = "width: %spx;"
CHECKBOX = "width: 25px;"


class ILastPool(compy.Interface):
    """A marker interface which specifies a 2-tuple of (last-pool,
    last-index-name).
    """


def fileSpam(context, data):
    from quotient.web import webmail
    ava = context.locate(IResource).avatar
    from quotient.cleanup import ISpambayes
    try:
        spam = ISpambayes(ava)
    except compy.CannotAdapt:
        return span(_class="disabled")['Spam filtering not installed.']

    spamPool = spam.spamPool
    cleanPool = spam.cleanPool

    if data is spamPool:
        pl = 'Clean'
        style = 'file-clean'
    # double BARF
    elif data.__class__ is EmailPool:
        pl = 'Spam'
        style= 'file-spam'
    else:
        return ''

    try:
        guesses = context.locate(webmail.ISpamGuesses)
    except KeyError:
        top = context
        while top.parent is not None:
            top = top.parent
        guesses = {}
        top.remember(guesses, webmail.ISpamGuesses)
    items = guesses.get(pl,[])
    if items:
        return href(spam, file=pl, items=items)["File %s" % pl]
    return span(_class="link-disabled")[ "File %s" % pl ]




def Method(name, arguments=[]):
    return formless.MethodBinding(
        name,  ## The name of an action
        formless.Method(
            None,
            arguments,
            label = formless.labelAndDescriptionFromNameAndDocstring(name, '')
        )
    )



def Action(name):
    return Method(
        name,
        [formless.Argument('items', formless.List(formless.Object()))]
    )


from quotient.web import filer


# a jumper is a link which lets you jump N items ahead
NUMBER_OF_JUMPERS = 5


class ListItems(formless.Typed):
    __implements__ = formless.Typed.__implements__, formless.iformless.IInputProcessor

    def process(self, ctx, boundTo, data):
        print "DATA", data
        return data


class ISelf(formless.TypedInterface):
    self = formless.Object()


class PoolBrowser(root.Base, configurable.Configurable):
    __implements__ = (root.Base.__implements__,
        configurable.Configurable.__implements__,
        IRendererFactory, ISelf, inevow.IRenderer)

    suggestedIndex = 'uid'
    suggestedBackwards = False
    isPool = False
    itemsPerPage = 25
    showInvisible = True
    def __init__(self, original, name='', renderAddPool=False, suggestedView=None):
        """Name: A unique name for this PoolBrowser which will identify it on a page with
        other embedded PoolBrowsers; it will also be shown to the user to identify it
        """
        self.name = name
        if not isinstance(original, list):
            root.Base.__init__(self, original)
            self.suggestedIndex = original.suggestedIndex
            self.suggestedBackwards = original.suggestedBackwards
            self.isPool = True
        configurable.Configurable.__init__(self, original)
        self.renderAddPool = renderAddPool
        self.suggestedView = suggestedView

    self = property(lambda self: self)

    def bind_poolAction(self, ctx):
        return formless.MethodBinding('poolAction', 
            formless.Method(
                None, [formless.Argument('ctx', formless.Context()), 
                    formless.Argument('action', formless.String()), 
                    formless.Argument('selection', ListItems())]))

    def poolAction(self, ctx, action, selection):
        pool = self.original
        get = pool.store.getItemByID
        try:
            selection = [get(int(x)) for x in selection]
        except:
            return "Error, invalid input"
        if action == 'label':
            labelName = ctx.arg('target')
            labels = ctx.locate(application.IDivmodApp).labels
            for target in labels.queryIndex('name', startKey=labelName, endKey=labelName):
                ## Label name already exists.
                break
            else:
                ## Label name does not exist, create it.
                target = store.Pool(self.original.store, name=labelName)
                labels.addItem(target)
            for I in selection:
                if I not in target:
                    target.addItem(I)
            return "Items labeled %r." % (target.name, )
        for I in selection:
            item = iquotient.ITabularRenderer(I)
            A = getattr(item, 'action_%s' % (action, ))
            A(pool)
        return action

    def render_anItem(self, context, data):
        ## Put the tabular pool instance, which was passed to us in data, on the stack as a renderer
        ## so we can find render_ methods
        self.even = even = not getattr(self, 'even', False)

        context.remember(data, IRendererFactory)

        selectedColumn = self.selectedColumn
        item = data.original
        #if hasattr(data, 'getCellClass'):
        #    cellclass = data.getCellClass(context, item)
        #else:
        #    cellclass=''

        K = context.key
        if K is None: K = ''
        if hasattr(item, 'storeID'):
            checkboxId = item.storeID
        else:
            checkboxId = 0
        if self.isPool:
            actions = bool(self.actionCollector)
        else:
            actions = False

        return table(
                _class=("list-row", even and '-even' or ''))[
            tr[
                td(style=CHECKBOX)[
                    not actions and img(
                        src="/images/small-icons/%s.png" % item.__class__.__name__
                    ) or input(
                        type="checkbox",
                        name = "selection",
                        value = checkboxId)
                ],
                [
                    td(style=width != 0 and 'width: %spx' % (width, ) or None,
                        _class=column == selectedColumn and 'cell-selected' or 'cell-normal')[
                            hasattr(data, 'render_%s' % column) and invisible(renderer=directive(column), data=item) or ' '  
                    ]
                    for position, column, width, displayName in self.columnCollector
                ],
            ]
        ]


    def getPrefs(self, context):
        ava = context.locate(credup.IAvatar)
        prefs = IPrefs(ava)
        self.itemsPerPage = itemsPerPage = prefs.getPref('itemsPerPage', 25)
        self.showInvisible = showInvisible = prefs.getPref('showInvisible')

    def locateIndexIterator(self, context, data):
        request = context.locate(IRequest)
        self.getPrefs(context)
        if not self.isPool:
            start = request.args.get('start', ['0'])[0]
            try:
                start = int(start)
            except:
                start = 0

            self.indx = [TabularAggregator(item) for item in self.original[start:self.itemsPerPage+start]]
            ## self.indx is a list of TabularRenderers
            ## And here is where we figure out if we can batch this list
            if start == 0:
                prev = 'Previous'
            else:
                prev = a(href=gethere(start=max(0, start-self.itemsPerPage)))["Previous"]
            if len(self.original)>self.itemsPerPage + start:
                next = a(href=gethere(start=min(len(self.original), self.itemsPerPage+start)))["Next"]
            else:
                next = "Next"
            nextPrev = lambda headerWidth: div(_class="batching-line", style="width: %spx" % headerWidth)[ span(_class="batching-prev") [ prev ], span(_class="batching-next") [ next ], xml('&nbsp;'), taglib.br ]

            return self.indx, nextPrev
        defaultIndex = self.suggestedIndex
        index = request.args.get('index', [defaultIndex])[0]
        self.index = index
        self.selectedColumn = index
        if index == defaultIndex:
            defaultQueryBack = self.suggestedBackwards
        else:
            defaultQueryBack = False

        queryBack = request.args.get('direction',
            [defaultQueryBack and 'backwards' or 'forewards'])[0] == 'backwards'

        if not self.isPool or not data.indexes.has_key(index):
            index = 'uid'
        self.queryBack = queryBack

        ssk = request.args.get('start', [None])[0]
        startItemID = 0
        if ssk:
            startKey = data.indexes[index].sortableStringToKey(ssk)
            startItemID = int(request.args.get('startid',[0])[0])
        else:
            startKey = None

        sek = request.args.get('end', [None])[0]
        if sek:
            endKey = data.indexes[index].sortableStringToKey(sek)
        else:
            endKey = None

        request.getSession().setComponent(ILastPool, (data, index, queryBack))
        indexIterator = data.queryIndex(index, startKey=startKey, endKey=endKey, backwards=queryBack, startItemID=startItemID)

        self.indx = indx = []
        for item in islice(indexIterator, 0, self.itemsPerPage):
            renderer = TabularAggregator(item)
            indx.append(renderer)
        
        JUMP_STRIDE = self.itemsPerPage
        forwardJumps = []
        for nextMessage in indexIterator:
            startstring = data.indexes[index].keyToSortableString(nextMessage.getIndexKey(data, index))
            startidstring = str(nextMessage.storeID)
            next = a(href=here(direction=queryBack and 'backwards' or 'forewards', start=startstring, index=index, startid=startidstring))['Next']
            forwardJumps.append(nextMessage)
            jumpCount = 0
            for jumpForward in indexIterator:
                jumpCount += 1
                if jumpCount >= JUMP_STRIDE:
                    forwardJumps.append(jumpForward)
                    jumpCount = 0
                    if len(forwardJumps) >= NUMBER_OF_JUMPERS: break
            break
        else:
            forwardJumps = []
            next = span(_class='disabled')['Next']

        backwardJumps = []
        if startKey is None:
            bList = []
        else:
            backIter = data.queryIndex(index, result=STORE_ID, startKey=startKey, startItemID=startItemID, backwards=not queryBack, limit=self.itemsPerPage+1)
            bList = list(backIter)
        # make sure we're not showing the first page
        if bList and ((not self.indx) or (bList[-1] != self.indx[0].original.storeID)):
            sid = bList[-1]
            ava = context.locate(credup.IAvatar)
            obj = ava.getItemByID(sid)
            endstring = data.indexes[index].keyToSortableString(obj.getIndexKey(data, index))
            prev = a(href=here(direction=queryBack and 'backwards' or 'forewards', start=endstring, startid=sid, index=index))['Previous']
            jumpCount = 0
            backwardJumps.append(obj)
            for jumpBack in data.queryIndex(index, startKey=obj.getIndexKey(data, index), startItemID=obj.storeID, backwards=not queryBack):
                jumpCount += 1
                if jumpCount >= JUMP_STRIDE:
                    backwardJumps.append(jumpBack)
                    jumpCount = 0
                    if len(backwardJumps) >= NUMBER_OF_JUMPERS:
                        break                    
            backwardJumps.reverse()
        else:
            prev = span(_class='disabled')["Previous"]

        total = self.original.indexCounter[index]

        def toNav(jumps, label):
            return [
                a(
                    _class="batch-jumper-link",
                    href=here(
                        direction=queryBack and 'backwards' or 'forewards',
                        start=data.indexes[index].keyToSortableString(obj.getIndexKey(data,index)),
                        startid=obj.storeID, index=index))[
                            label ]
            for obj in jumps
            ]

        nextPrev = lambda headerWidth: div(_class="batching-line", style="width: %100")[
            span(_class="batching-prev") [ prev ], 
            span(_class="batching-next") [ next ],
            toNav(backwardJumps, '<'),
            xml('&nbsp;'),
            toNav(forwardJumps, '>'),
        ]

        return indexIterator, nextPrev

    nextPrev = ''
    def collectColumns(self, indx):
        self.columnCollector = columnCollector = []
        seenCols = {}
        self.fullHeaderWidth = 25
        for item in indx:
            accumulator = {}
            reflect.accumulateMethods(item, accumulator, 'render_')
            for name, method in accumulator.items():
                position = getattr(method, 'position', False)
                if position is False:
                    continue
                if getattr(method, 'default', False):
                    if not seenCols.has_key(name):
                        seenCols[name] = True
                        columnCollector.append((position, name, method.width, method.colname))
                        self.fullHeaderWidth += method.width
        columnCollector.sort()

    def collectActions(self, indx):
        self.actionCollector = actionCollector = []
        seenActions = {}
        for item in indx:
            accumulator = {}
            reflect.accumulateMethods(item, accumulator, 'action_')
            for name, method in accumulator.items():
                if not seenActions.has_key(name):
                    seenActions[name] = True
                    actionCollector.append(name)
        actionCollector.sort()

    def calculateHeaders(self, index='', queryBack=False, item=None, indexes={}):
        actions = []
        if self.isPool:
            actions = self.actionCollector
        row = tr[td(style=CHECKBOX, _class='pool-header')[
            actions == [] and xml('&nbsp;') or input(
                type="checkbox", onclick="""var inputs = document.getElementsByTagName('input');
for (i=0; i<inputs.length; i++) {
    if (inputs[i].name == 'selection') {
        inputs[i].checked = this.checked;
    }
}""")]]

        self.headers = table(_class="list-row")[row]
        for position, column, width, displayName in self.columnCollector:
            if column == index:
                klas = 'pool-header-highlight'
                direction = queryBack and 'forewards' or 'backwards'
                extra = img(src="/images/icons/%s.png" % (queryBack and 'backwards' or 'forwards'))
            else:
                klas = 'pool-header'
                direction = queryBack and 'backwards' or 'forewards'
                extra = ''
            if item is not None and indexes.has_key(column):
                content = href(
                    item, index=column, direction=direction, start=''
                )(id='TableHeader.' + displayName)[
                    displayName, extra]
            else:
                content = displayName
            row[
                td(style=width != 0 and CRAP % width or None, _class=klas)[content]
            ]

    def preparePool(self, context, data):
        indexIterator, nextPrev = self.locateIndexIterator(context, data)
        index = self.index
        indx = self.indx
        itemsPerPage = self.itemsPerPage
        showInvisible = self.showInvisible
        queryBack = self.queryBack


        self.collectColumns(indx)
        columnCollector = self.columnCollector

        self.collectActions(indx)
        actionCollector = self.actionCollector

        self.calculateHeaders(index, queryBack, data, data.indexes)
        headers = self.headers

        if showInvisible:
            contentsList = indx
        else:
            contentsList = [item for item in indx if not item.invisible]
        if contentsList:
            contentsList = [
                div(renderer=directive("anItem"), data=item) for item in contentsList
            ]
        else:
            contentsList = [ p["No items."] ]
            self.fullHeaderWidth = 500
            headers = ''

        self.nextPrev = nextPrev = nextPrev(self.fullHeaderWidth)

        ## set ivars based on what we just figured out from looking at the pool
        hd = lambda **kw: div(style="width: 100%", **kw)
        self._header = self.name, hd()[ headers ]
        self._contents = contentsList
            ##div(renderer=directive("anItem"), data=item) for item in contentsList
#        ]
        self._footer = nextPrev

    def render_pool(self, context, data):
        if isinstance(data, EmailPool):
            data.setUIDMark()
        if self.isPool:
            self.preparePool(context, self.original)
            storeId = self.original.storeID
        else:
            storeId = 0
            self.actionCollector = []
            self._footer = ""
            self.selectedColumn = ''
            self.fullHeaderWidth = 500
            ## do the slicing and batching here
            self.getPrefs(context)
            ipp = self.itemsPerPage
            renderers = [TabularAggregator(item) for item in islice(self.original, 0, ipp)]
            self._contents = [
                div(renderer=directive("anItem"), data=item) for item in renderers
            ]
            self.collectColumns(renderers)
            self.collectActions(renderers)
            self.calculateHeaders()
            self._header = (self.name, self.headers)

        footer = self._footer
        ## triple BARF
        for item in self._contents:
            if hasattr(getattr(item, 'data', None), 'render_bayes'):
                footer = (div(_class="actions", data=self.original)[fileSpam], footer)
                break

        if isinstance(self.original, EmailPool):
            labels = context.locate(application.IDivmodApp).labels
            labelsList = [x.name for x in labels.queryIndex('name')]
            filer = select(name="target", onchange="""node = this.childNodes[this.selectedIndex];
value = node.value;

if (value=='  **_**  ') {
    node.value = prompt('New label name', '');
}
if (value!='  **x**  ') {
    this.parentNode.parentNode.submit();
}
""")[
                option(value="  **x**  ")["Apply label"],
                [option(value=x)[x] for x in labelsList],
                option(value="  **_**  ")["New label..."]], input(type="hidden", name="action", id="labelAction", value="label")
        else:
            filer = ''
        if self.isPool:
            actions = self.actionCollector
            yield h1[self.original.name]
        else:
            actions = []
        yield form(action=(storeId, '/freeform_post!!poolAction'), key="Table", _class="pool-browser-form")[
            self.nextPrev,
            div(_class="actions")[
                [input(type="submit", name="action", value=str(action)) for action in actions],
                filer,
            ],
            div(_class="pool-browser")[
                self._header, 
                self._contents
            ],
            footer]

    ## When used as a Resource, this document attribute will be used
    document = taglib.page(title=name)[ render_pool ]

    ## When an instance of PoolBrowser is placed in a page,
    ## __call__ will be used
    def rend(self, context, data):
        context.remember(self, IRendererFactory)
        return invisible(data=self)[comment['\nPoolBrowser.rend\n'], self.render_pool]

