# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

import sre
import os
import random
import datetime
from StringIO import StringIO

from quotient.taglib import *
from quotient import taglib

from twisted.python import components
from twisted.internet import threads
from twisted.web.microdom import parseString
from twisted.web import static

from atop import sched
from atop import tpython
from atop import powerup

from quotient.web.scrubber import scrub
from quotient.web import root
from quotient.web.root import column, action_delete
from quotient import timezones
from quotient import prefup
from quotient import mimemessage

from nevow.rend import RenderFactory
from nevow.tags import *
from nevow.inevow import IRequest, IResource
from quotient.iquotient import ITabularRenderer
from quotient.proto.smtpout import OutgoingMailup
from formless import webform
from nevow import blocks
from atop import credup

from formless import annotate

class ISpamGuesses(components.Interface):
    """A dict of lists of storeids in the current web page
    and what the guess was. Like

    {'spam': [1,2,3], 'clean': [4,5,6]}
    """


urlRe = sre.compile(r'(\b(?:http|ftp|https)://\S+(?:\b|/)?|\b[-.\w]+@[-.\w]+)')


def render_received(self, context, data):
    dr = data.dateReceived
    # convert to datetime object
    local = tpython.mkgmtime(dr)
    dtobj = datetime.datetime.fromtimestamp(local, timezones.Local)
    # and here's where we have to respect the user's account setting, hoorj
    astz = dtobj.astimezone(prefup.IPrefs(data.store).timezone)
    now = datetime.datetime.now(timezones.Local)
    if astz.date() == now.date():
        return astz.strftime('%H:%M %p').lower()
    elif astz.date().year == now.date().year:
        res = astz.strftime('%b, %H:%M %p').lower().capitalize()
        res = str(astz.date().day) + ' ' + res
        return res
    else:
        res = astz.strftime('%b %y, %H:%M %p').lower().capitalize()
        res = str(astz.date().day) + ' ' + res
        return res


class TabularMessage(components.Adapter, RenderFactory):
    invisible = False
    def getCellClass(self, context, msg):
        if msg.getIndexKey(None, 'flags', 0) & msg.SEEN:
            return ""
        return "cell-new"

    def render_flags(self, context, message):
        ava = context.locate(credup.IAvatar)

        prefs = prefup.IPrefs(ava)

        accountColors = prefs.getPref('accountColors', {})

        rv = message.receivedVia
        thenode = context.tag
        flags = message.getIndexKey(None, 'flags', 0)
        if not flags & message.SEEN:
            if accountColors.has_key(rv):
                colorNum = accountColors[rv]
            else:
                colorNum = len(accountColors) + 1
                while colorNum >= 10:
                    colorNum %= 10
                accountColors[rv] = colorNum
                prefs.touch()
            thenode[ img(src="/images/flags/new_%s.png" % colorNum, title=rv) ]
        for attach in message.getAttachments():
            break
        else:
            attach = None
        if attach is not None:
            thenode[ img(src="/images/flags/attachments.png") ]
        if flags & message.ANSWERED:
            thenode[ img(src="/images/flags/replied.png", title=rv) ]
        if flags & message.FORWARDED:
            thenode[ img(src="/images/flags/forwarded.png") ]
        if flags & message.REDIRECTED:
            ## todo create flags png for this
            thenode[ img(src="/images/toolbar/redirect.png") ]
        return thenode
    column(render_flags, True, 60)

    def render_sender(self, context, data):
        h = data.getIndexKey(None,'person')
        d = headerToUTF8(data.nameAndAddress()[0])
        if h:
            return href(h)(id=context.key + ".Sender")[d]
        else:
            return d
    column(render_sender, True, 160)

    def render_bayes(self, context, data):
        from quotient.cleanup import ISpambayes
        try:
            bayes = context.locate(ISpambayes)
        except KeyError:
            top = context
            while top.parent is not None:
                top = top.parent
            ava = context.locate(credup.IAvatar)
            bayes = ISpambayes(data.store)
            top.remember(bayes, ISpambayes)
        pool, prob = bayes.guessPool(data)

        if pool is None:
            # Convention is that if the pool is None
            # the message was routed by rule, the
            # name of the rule is in prob
            return [
                span(_class='fact') [ 'WL' ],
                ]
        try:
            guesses = context.locate(ISpamGuesses)
        except KeyError:
            top = context
            while top.parent is not None:
                top = top.parent
            guesses = {}
            top.remember(guesses, ISpamGuesses)

        guesses.setdefault(pool, []).append(data.storeID)
        currentGuessStyle = {
            'Clean': 'file-clean',
            'Spam': 'file-spam',
            'Unknown': 'spam-unknown'
        }[pool]

        return [href(bayes, spam=data.storeID)(_class="train-as-spam")["<"],
                span(_class=currentGuessStyle) [ '%0.2f' % prob ],
                href(bayes, clean=data.storeID)(_class="train-as-clean")[">"]]

    column(render_bayes, True, 65, "Spam?")

    def render_subject(self, context, data):
        subj = data['subject'] or ''
        if not subj.strip():
            subj = "<no subject>"
        return href(data)(id=context.key + ".Subject")[comment['subject'], headerToUTF8(subj)]
    column(render_subject, True, 0)

    render_name = None # don't render a name!!

    render_received = render_received
    column(render_received, True, 125, 'Date')

    def render_size(self, context, data):
        from quotient import stats
        if data.size <= 1024 * 1024:
            return stats.formatSize(data.size, 'B', 'KMGTP', False, False)
        return stats.formatSize(data.size, 'B', 'KMGTP', False)
    column(render_size, True, 60)

    action_delete = action_delete

    def action_MarkUnread(self, pool):
        self.original.changeFlags(None, self.original.SEEN, set=False)

    def action_MarkRead(self, pool):
        self.original.changeFlags(None, self.original.SEEN)


def viewedMessage(context, data):
    data.changeFlags(None, mimemessage.MIMEMessage.SEEN)
    return ""

def messageSource(context, data):
    label = 'View Message Source'
    return href(data, source="1")(title=label)[
        img(src="/images/toolbar/viewRaw.png")
    ]


MAX_WIDTH = 80

def appendButton(req, stan):
    B = getattr(req, 'extraButtons', [])
    req.extraButtons = B
    B.append(stan)

def messageBody(context, data):
    req = context.locate(IRequest)
    if req.args.get('source'):
        return div(_class="message-body")[pre[data.openFile().read()]]

    try:
        part = data.getDisplayPart()
    except ValueError:
        dom = span(_class="disabled")['No text in body.']
    else:
        partdata = part.get_payload(decode=True)
        ctypep = part.getHeaderParams('content-type')
        if ctypep.get('charset'):
            try:
                partdata = partdata.decode(ctypep['charset'], 'replace').encode('utf8')
            except LookupError, le:
                partdata = '%s:\n%s' % (le, partdata)

        if 'html' in part.get_content_type().lower():
            try:
                dom = parseString(partdata, beExtremelyLenient=True)
            except:
                return div(_class="message-body")['Garbage HTML, sorry.']
            if req.args.get('trusted'):
                text = "Don't trust this HTML."
                appendButton(req,
                             href(data)(title=text)[img(src="/images/toolbar/dont-trust-html.png", alt=text, title=text)]
                             )
                dom = xml(dom.toprettyxml())
            else:
                scrub(dom)
                text = "Trust this HTML."
                appendButton(req,
                    href(data, trusted="1")(title=text)[img(src="/images/toolbar/trust-html.png", alt=text, title=text)]
                )
                dom = xml(dom.toprettyxml())
        else:
            ## Split on newlines and shorten lines longer than 80 characters
            spl = partdata.split('\n')
            for i, v in enumerate(spl):
                if len(v) > MAX_WIDTH:
                    spl[i] = L = [v[:MAX_WIDTH]]
                    rest = v[MAX_WIDTH:]
                    while len(rest) > MAX_WIDTH:
                        L.append(rest[:MAX_WIDTH])
                        rest = rest[MAX_WIDTH:]
                    L.append(rest)
                else:
                    spl[i] = [v]
            spl = ['\n'.join(x) for x in spl]

            dom = pre['\n'.join(spl)]
    return div(_class="message-body")[dom, div(style="width: 300px; margin-bottom: 20px;")]

def messageExtracts(context, data):
    ava = context.locate(credup.IAvatar)
    result = []
    types = {}
    if hasattr(data, 'extracts'):
        for extract in data.extracts:
            extract = extract.getItem()
            if extract.uninteresting:
                continue
            types.setdefault(
                type(extract),
                []
            ).append(
                p['...', extract.toStan(), '...']) # quick hack
    return span(id="THECURRENT"), div(_class="absoluteRight")[[
        img(
            title="Found %s %ss." % (len(v), k.__name__),
            src="/images/extracts/%s.png" % k.__name__,
            onclick="""
var theCurrent = document.getElementById('THECURRENT');
if (theCurrent.className) {
    document.getElementById(theCurrent.className).style.display = 'none';
}
if (theCurrent.className != '%(name)s') {
    document.getElementById('%(name)s').style.display = 'block';
    theCurrent.className = '%(name)s';
} else {
    theCurrent.className = '';
}
""" % {'name': k.__name__},
            _class="extract-button")
        for k, v in types.items()
    ]], [
        div(
            id=k.__name__,
            _class="extract-box")
        [
            strong[k.__name__, len(v) > 1 and 's' or ''],
            v
        ]
        for k, v in types.items()
    ]

from quotient.web.pools import ILastPool
from atop.store import STORE_ID

def _link(text, previous):
    def _(context, data):
        req = context.locate(IRequest)
        # figure out which pool and index I'm iterating.
        p = req.getSession().getComponent(ILastPool, default=None)
        if p is None or data not in p[0]:
            pk = data.pools.keys()[0]
            pool = data.store.getItemByID(pk)
            indexName = pool.suggestedIndex
            lbackwards = pool.suggestedBackwards
        else:
            pool, indexName, lbackwards = p
        # TODO: distinction between index prefix and index name??
        skey = data.getIndexKey(pool, indexName)
        q = pool.queryIndex(indexName, startKey=skey, startItemID=data.storeID,
                            backwards=(previous ^ lbackwards)) #, result=STORE_ID
        for item in q:
            break
        for item in q:
            return href(item)(title=text)[text]
        else:
            return span(_class="disabled")[text]
    return _


def _action(text):
    def _(context, data):
        ava = context.locate(credup.IAvatar)
        outgoing = powerup.IPowerStation(ava).installPowerup(OutgoingMailup)
        return href(outgoing, **{text: data.storeID})(title=text, style="vertical-align: middle")[
            img(style="margin: 0px", src="/images/toolbar/%s.png" % text, alt=text)]
    return _


MODULE_NOT_PRESENT = "bad"


def attachments(context, msg):
    attList = list(msg.getAttachments())
    if attList:
        rv = div(_class="attachmentList")["Attachments"]
        for att in attList:
            rv[
                div(_class="attachmentButton")[
                    href(msg, "attachment", str(att.mimeID), "view", att.getAttachmentName())[
                        att.getAttachmentName()],
                    ' ',
                    href(msg, "attachment", str(att.mimeID), "download", att.getAttachmentName())[
                        "[download]"]]]
            inf = att.inferType()
            if inf.startswith('image'):
                rv[
                    href(msg, "attachment", att.mimeID, "view", att.getAttachmentName())[
                        img(src='%s/thumbsup:%s.png' % (msg.storeID, att.mimeID))]]
        return rv
    else:
        return ''


def From(context, data):
    pRef = data.getIndexKey(None, 'person')
    if pRef:
        return href(pRef)[messageHeader('From')]
    else:
        return messageHeader('From')

From.name = 'From'


def extra(context, data):
    req = context.locate(IRequest)
    return getattr(req, 'extraButtons', '')


prevnext = div(_class="batching-line")[
    span(_class="batching-next")[_link("Next", False)],
    span(_class="batching-prev")[_link("Previous", True)], xml('&nbsp;')]



class IMessage(annotate.TypedInterface):
    def removeLabel(self, context=annotate.Context(), label=annotate.String()):
        """Remove a label
        """
    removeLabel = annotate.autocallable(removeLabel)


class MessageRenderer(root.Base):
    __implements__ = IMessage, root.Base.__implements__

    document = page(title=(messageHeader('subject')))[
            div(_class="toolbar-line")[
                _action("reply"),
                _action("forward"),
                _action("redirect"),
                extra,
                messageSource],
            div(style="margin-left: 115px;  border-left: 1px solid black;")[
                prevnext,
                headers(From,'To','Cc','Date','Subject'),
                attachments,
                messageBody,
                taglib.containerList,
                prevnext,
                viewedMessage,
                ]]

    def getDynamicChild(self, path, request):
        if path.startswith('thumbsup:'):
            tmpdir = self.original.newCacheDir('thumbsup')
            partid = path.split('.')[0].split(':')[-1]
            p = self.original.getPartByID(int(partid))
            imgName = os.path.join(tmpdir, '%s.png' % partid)
            if not os.path.exists(imgName):
                return missingImage()
            return static.File(imgName, p.inferType())

    def child_attachment(self, request):
        return AttachmentLoader(self.original)

    def removeLabel(self, context, label):
        store = self.original.store
        try:
            L = store.getItemByID(int(label))
        except Exception, e:
            return "Invalid input"
        from quotient import application
        labels = context.locate(application.IDivmodApp).labels
        if L not in labels:
            return "Can't remove that label."
        L.removeItem(self.original)


class AttachmentLoader(components.Adapter):
    __implements__ = IResource
    getComponent = None

    def locateChild(self, request, segments):
        number, mode, filename = segments[:3]
        i = int(number)
        return PartData(self.original.getPartByID(i), mode == 'attachment', filename), segments[3:]


class PartData:
    __implements__ = IResource
    def __init__(self, part, attachment, filename):
        self.part = part
        self.attachment = attachment
        self.filename = filename

    def locateChild(self, request, segments):
        return self, ()

    def renderHTTP(self, request):
        request.setHeader('content-type', self.part.get_type())
        if self.attachment:
            request.setHeader('content-disposition',
                              'attachment; filename=%r' % (self.filename))
        return self.part.get_payload(True)

def render_nextrun(self, context, data):
    now = sched.IScheduler(data.store).clock()
    then = data.getIndexKey(None, "scheduled")
    if then is None:
        return "(Not Running)"
    return tpython.humanInterval(then - now)

class ScheduledEventTabularRenderer(root.TabularBase):

    invisible = False

    def render_name(self, context, data):
        return href(data)[data.name or data.__class__.__name__]
    column(render_name, True, 200)

    render_nextrun = render_nextrun
    column(render_nextrun, True, 200, "Next Run In")

    def render_status(self, context, data):
        if self.original.status:
            return img(
                src="/images/icons/generic.png", title=str(self.original.status)
            ), href(self.original, "freeform_post!!check")[ 'Check now' ]
        return 'N/A'
    column(render_status, True, 200)

    def action_Delete(self, pool):
        rv = "%s deleted." % self.original
        self.original.delete()
        return rv


class ScheduledEventDisplay(root.Base):

    document = page(title="Event")[
        p["A scheduled event: ", str],
        p["Next run: ", render_nextrun],
        webform.renderForms()
    ]
